@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -17,11 +14,6 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* draw_radiation.c
- *
- * Code for drawing radiation patterns
- */
-
 #include "draw_radiation.h"
 #include "shared.h"
 
@@ -34,54 +26,47 @@ static point_3d_t *point_3d = NULL;
 
 /*-----------------------------------------------------------------------*/
 
-/* Draw_Radiation()
+/* Scale_Gain()
  *
- * Draws the radiation pattern or near E/H fields
+ * Scales radiation pattern gain according to selected style
+ * ( ARRL style, logarithmic or linear voltage/power )
  */
-  void
-Draw_Radiation( GtkWidget *drawingarea )
+static int gain_style = GS_LINP;
+
+  static double
+Scale_Gain( double gain, int fstep, int idx )
 {
-  /* Block motion events */
-  g_signal_handler_block( (gpointer)drawingarea, rdpattern_motion_handler );
+  /* Scaled rad pattern gain and pol factor */
+  double scaled_rad = 0.0;
 
-  /* Abort if xnec2c may be quit by user */
-  if( isFlagSet(MAIN_QUIT) || isFlagClear(ENABLE_EXCITN) )
-	return;
+  gain += Polarization_Factor( calc_data.pol_type, fstep, idx );
 
-  /* Cairo context */
-  cairo_t *cr = gdk_cairo_create( rdpattern_pixmap );
+  switch( gain_style )
+  {
+	case GS_LINP:
+	  scaled_rad = pow(10.0, (gain/10.0));
+	  break;
 
-  /* Clear pixmap */
-  cairo_set_source_rgb( cr, BLACK );
-  cairo_rectangle(
-	  cr, 0.0, 0.0,
-	  (double)rdpattern_proj_params.pixmap_width,
-	  (double)rdpattern_proj_params.pixmap_height);
-  cairo_fill( cr );
+	case GS_LINV:
+	  scaled_rad = pow(10.0, (gain/20.0));
+	  break;
 
-  /* Draw rad pattern or E/H fields */
-  if( isFlagSet(DRAW_GAIN) )
-	Draw_Radiation_Pattern();
-  else if( isFlagSet(DRAW_EHFIELD) )
-	  Draw_Near_Field();
+	case GS_ARRL:
+	  scaled_rad = exp( 0.058267 * gain );
+	  break;
 
-  /* Render pixmap to screen */
-  gdk_window_set_back_pixmap( drawingarea->window,
-	  rdpattern_pixmap, FALSE );
-  gdk_window_clear( drawingarea->window );
+	case GS_LOG:
+	  scaled_rad = gain;
+	  if( scaled_rad < -40 )
+		scaled_rad = 0.0;
+	  else
+		scaled_rad = scaled_rad /40.0 + 1.0;
 
-  /* Display frequency step */
-  Display_Fstep( rdpattern_fstep_entry, calc_data.fstep );
+  } /* switch( gain_style ) */
 
-  /* Wait for GTK to complete its tasks */
-  while( g_main_context_iteration(NULL, FALSE) );
+  return( scaled_rad );
 
-  cairo_destroy( cr );
-
-  /* Unblock motion events */
-  g_signal_handler_unblock( (gpointer)drawingarea, rdpattern_motion_handler );
-
-} /* Draw_Radiation() */
+} /* Scale_Gain() */
 
 /*-----------------------------------------------------------------------*/
 
@@ -91,17 +76,18 @@ Draw_Radiation( GtkWidget *drawingarea )
  * segmants joining the points defined by spherical
  * co-ordinates theta, phi and r = gain(theta, phi)
  */
-  void
-Draw_Radiation_Pattern( void )
+  static void
+Draw_Radiation_Pattern( cairo_t *cr )
 {
   /* Abort if rad pattern cannot be drawn */
   if( isFlagClear(ENABLE_RDPAT) || (calc_data.fstep < 0) )
 	return;
 
   /* Line segments to draw on Screen */
-  GdkSegment segm;
+  Segment_t segm;
 
   int
+	idx,
 	nth,     /* Theta step count */
 	nph,     /* Phi step count   */
 	col_idx, /* Index to rad pattern color buffers */
@@ -116,14 +102,11 @@ Draw_Radiation_Pattern( void )
   double theta, phi, r, r_min, r_range;
 
   /* theta and phi step in rads */
-  double dth = (double)fpat.dth * (double)TA;
-  double dph = (double)fpat.dph * (double)TA;
+  double dth = (double)fpat.dth * (double)TORAD;
+  double dph = (double)fpat.dph * (double)TORAD;
 
   /* Used to set text in labels */
-  gchar txt[7];
-
-  /* Cairo context */
-  cairo_t *cr = gdk_cairo_create( rdpattern_pixmap );
+  gchar txt[8];
 
   fstep = calc_data.fstep;
   pol   = calc_data.pol_type;
@@ -132,50 +115,49 @@ Draw_Radiation_Pattern( void )
   if( isFlagSet(DRAW_NEW_RDPAT) )
   {
 	size_t mreq = ((size_t)(fpat.nth * fpat.nph)) * sizeof(point_3d_t);
-	mem_realloc( (void *)&point_3d, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&point_3d, mreq, "in draw_radiation.c" );
 	mreq = (size_t)((fpat.nth-1) * fpat.nph + (fpat.nph-1) * fpat.nth);
 	mreq *= sizeof(double);
-	mem_realloc( (void *)&red, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&grn, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&blu, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&red, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&grn, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&blu, mreq, "in draw_radiation.c" );
 
 	ClearFlag( DRAW_NEW_RDPAT );
 
 	/* Distance of rdpattern point furthest from xyz origin */
+	idx = rad_pattern[fstep].max_gain_idx[pol];
 	rdpattern_proj_params.r_max = Scale_Gain(
-		rad_pattern[fstep].gtot[rad_pattern[fstep].max_gain_idx[pol]],
-		fstep, rad_pattern[fstep].max_gain_idx[pol]);
+		rad_pattern[fstep].gtot[idx], fstep, idx);
 
 	/* Distance of rdpattern point nearest to xyz origin */
+	idx = rad_pattern[fstep].min_gain_idx[pol];
 	r_min = Scale_Gain(
-		rad_pattern[fstep].gtot[rad_pattern[fstep].min_gain_idx[pol]],
-		fstep, rad_pattern[fstep].min_gain_idx[pol]);
+		rad_pattern[fstep].gtot[idx], fstep, idx);
 
 	/* Range of scaled rdpattern gain values */
 	r_range = rdpattern_proj_params.r_max - r_min;
 
 	/* Set radiation pattern projection parametrs */
 	New_Projection_Parameters(
-		rdpattern_pixmap_width,
-		rdpattern_pixmap_height,
+		rdpattern_width,
+		rdpattern_height,
 		&rdpattern_proj_params );
 
 	/*** Convert radiation pattern values
 	 * to points in 3d space in x,y,z axis ***/
 	pts_idx = 0;
-	phi = (double)(fpat.phis * TA); /* In rads */
+	phi = (double)(fpat.phis * TORAD); /* In rads */
 
 	/* Step phi angle */
 	for( nph = 0; nph < fpat.nph; nph++ )
 	{
-	  theta = (double)(fpat.thets * TA); /* In rads */
+	  theta = (double)(fpat.thets * TORAD); /* In rads */
 
 	  /* Step theta angle */
 	  for( nth = 0; nth < fpat.nth; nth++ )
 	  {
 		/* Distance of pattern point from the xyz origin */
-		r = Scale_Gain(
-			rad_pattern[fstep].gtot[pts_idx], fstep, pts_idx );
+		r = Scale_Gain( rad_pattern[fstep].gtot[pts_idx], fstep, pts_idx );
 
 		/* Distance of pattern point from xyz origin */
 		point_3d[pts_idx].r = r;
@@ -241,33 +223,29 @@ Draw_Radiation_Pattern( void )
 	} /* for( nph = 1; nph < fpat.nph; nph++ ) */
 
 	/* Show max gain on color code bar */
-	snprintf( txt, 7, "%6f", rad_pattern[fstep].max_gain[pol] );
-	txt[6] = '\0';
-	gtk_label_set_text(
-		GTK_LABEL(lookup_widget(rdpattern_window,
-			"rdpattern_colorcode_maxlabel")), txt );
+	snprintf( txt, 8, "%6.1f", rad_pattern[fstep].max_gain[pol] );
+	gtk_label_set_text(	GTK_LABEL(
+		  Builder_Get_Object(rdpattern_window_builder, "rdpattern_colorcode_maxlabel")),
+		txt );
 
 	/* Show min gain on color code bar */
-	snprintf( txt, 5, "%4f", rad_pattern[fstep].min_gain[pol] );
-	txt[6] = '\0';
-	gtk_label_set_text(
-		GTK_LABEL(lookup_widget(rdpattern_window,
-			"rdpattern_colorcode_minlabel")), txt );
+	snprintf( txt, 6, "%4.1f", rad_pattern[fstep].min_gain[pol] );
+	gtk_label_set_text(GTK_LABEL(
+		  Builder_Get_Object(rdpattern_window_builder, "rdpattern_colorcode_minlabel")),
+		txt );
 
   } /* if( isFlagSet(DRAW_NEWRDPAT) ) ) */
 
   /* Draw xyz axes to Screen */
-  Draw_XYZ_Axes( rdpattern_pixmap, rdpattern_proj_params );
+  Draw_XYZ_Axes( cr, rdpattern_proj_params );
 
   /* Overlay structure on Near Field pattern */
   if( isFlagSet(OVERLAY_STRUCT) )
   {
-	/* Save structure pixmap and params pointers */
-	GdkPixmap *pixmap = structure_pixmap;
+	/* Save structure projection params pointers */
 	projection_parameters_t params = structure_proj_params;
 
 	/* Divert structure drawing to rad pattern area */
-	structure_pixmap = rdpattern_pixmap;
 	structure_proj_params = rdpattern_proj_params;
 	structure_proj_params.r_max = params.r_max;
 	structure_proj_params.xy_scale =
@@ -276,12 +254,12 @@ Draw_Radiation_Pattern( void )
 	/* Process and draw geometry if enabled */
 	Process_Wire_Segments();
 	Process_Surface_Patches();
-	Draw_Surface_Patches( structure_segs+data.n, data.m );
-	Draw_Wire_Segments( structure_segs, data.n );
+	Draw_Surface_Patches( cr, structure_segs+data.n, data.m );
+	Draw_Wire_Segments( cr, structure_segs, data.n );
 
-	/* Restore structure pixmap and params */
-	structure_pixmap = pixmap;
+	/* Restore structure projection params */
 	structure_proj_params = params;
+
   } /* if( isFlagSet(OVERLAY_STRUCT) ) */
 
   /*** Draw rad pattern on screen ***/
@@ -343,10 +321,11 @@ Draw_Radiation_Pattern( void )
   } /* for( nth = 0; nth < fpat.nth; nth++ ) */
 
   /* Show gain in direction of viewer */
-  Show_Viewer_Gain( rdpattern_window,
-	  "rdpattern_viewer_gain", rdpattern_proj_params );
+  Show_Viewer_Gain(
+	  rdpattern_window_builder,
+	  "rdpattern_viewer_gain",
+	  rdpattern_proj_params );
 
-  cairo_destroy( cr );
 } /* Draw_Radiation_Pattern() */
 
 /*-----------------------------------------------------------------------*/
@@ -355,8 +334,8 @@ Draw_Radiation_Pattern( void )
  *
  * Draws near E/H fields and Poynting vector
  */
-  void
-Draw_Near_Field( void )
+  static void
+Draw_Near_Field( cairo_t *cr )
 {
   int idx, npts; /* Number of points to plot */
   double
@@ -378,18 +357,15 @@ Draw_Near_Field( void )
   gchar txt[9];
 
   /* Line segments to draw on Screen */
-  GdkSegment segm;
+  Segment_t segm;
 
   /* For coloring field lines */
-  double rd, gn, bl;
+  double xred = 0.0, xgrn = 0.0, xblu = 0.0;
 
   /* Abort if drawing a near field pattern is not possible */
   if( isFlagClear(ENABLE_NEAREH) || !near_field.valid )
 	return;
 
-  /* Cairo context */
-  cairo_t *cr = gdk_cairo_create( rdpattern_pixmap );
-
   /* Initialize projection parameters */
   if( isFlagSet(DRAW_NEW_EHFIELD) )
   {
@@ -407,8 +383,8 @@ Draw_Near_Field( void )
 	/* Distance of field point furthest from xyz origin */
 	rdpattern_proj_params.r_max = near_field.r_max + dr;
 	New_Projection_Parameters(
-		rdpattern_pixmap_width,
-		rdpattern_pixmap_height,
+		rdpattern_width,
+		rdpattern_height,
 		&rdpattern_proj_params );
 
 	ClearFlag( DRAW_NEW_EHFIELD );
@@ -416,27 +392,24 @@ Draw_Near_Field( void )
   } /* if( isFlagSet( DRAW_NEW_EHFIELD ) */
 
   /* Draw xyz axes to Screen */
-  Draw_XYZ_Axes( rdpattern_pixmap, rdpattern_proj_params );
+  Draw_XYZ_Axes( cr, rdpattern_proj_params );
 
   /* Overlay structure on Near Field pattern */
   if( isFlagSet(OVERLAY_STRUCT) )
   {
-	/* Save structure pixmap and params pointers */
-	GdkPixmap *pixmap = structure_pixmap;
+	/* Save projection params pointers */
 	projection_parameters_t params = structure_proj_params;
 
 	/* Divert structure drawing to rad pattern area */
-	structure_pixmap = rdpattern_pixmap;
 	structure_proj_params = rdpattern_proj_params;
 
 	/* Process and draw geometry if enabled */
 	Process_Wire_Segments();
 	Process_Surface_Patches();
-	Draw_Surface_Patches( structure_segs+data.n, data.m );
-	Draw_Wire_Segments( structure_segs, data.n );
+	Draw_Surface_Patches( cr, structure_segs+data.n, data.m );
+	Draw_Wire_Segments( cr, structure_segs, data.n );
 
-	/* Restore structure pixmap and params */
-	structure_pixmap = pixmap;
+	/* Restore structure params */
 	structure_proj_params = params;
   } /* if( isFlagSet(OVERLAY_STRUCT) ) */
 
@@ -448,7 +421,7 @@ Draw_Near_Field( void )
 	if( isFlagSet(DRAW_EFIELD) && (fpat.nfeh & NEAR_EFIELD) )
 	{
 	  /* Set gc attributes for segment */
-	  Value_to_Color( &rd, &gn, &bl,
+	  Value_to_Color( &xred, &xgrn, &xblu,
 		  near_field.er[idx], near_field.max_er );
 
 	  /* Scale factor for each field point, to make
@@ -470,7 +443,7 @@ Draw_Near_Field( void )
 		  fx, fy, fz );
 
 	  /* Draw segment */
-	  cairo_set_source_rgb( cr, rd, gn, bl );
+	  cairo_set_source_rgb( cr, xred, xgrn, xblu );
 	  Cairo_Draw_Line( cr, segm.x1, segm.y1, segm.x2, segm.y2 );
 
 	} /* if( isFlagSet(DRAW_EFIELD) && (fpat.nfeh & NEAR_EFIELD) ) */
@@ -479,7 +452,7 @@ Draw_Near_Field( void )
 	if( isFlagSet(DRAW_HFIELD) && (fpat.nfeh & NEAR_HFIELD) )
 	{
 	  /* Set gc attributes for segment */
-	  Value_to_Color( &rd, &gn, &bl,
+	  Value_to_Color( &xred, &xgrn, &xblu,
 		  near_field.hr[idx], near_field.max_hr );
 
 	  /* Scale factor for each field point, to make
@@ -501,7 +474,7 @@ Draw_Near_Field( void )
 		  fx, fy, fz );
 
 	  /* Draw segment */
-	  cairo_set_source_rgb( cr, rd, gn, bl );
+	  cairo_set_source_rgb( cr, xred, xgrn, xblu );
 	  Cairo_Draw_Line( cr, segm.x1, segm.y1, segm.x2, segm.y2 );
 
 	} /* if( isFlagSet(DRAW_HFIELD) && (fpat.nfeh & NEAR_HFIELD) ) */
@@ -512,15 +485,16 @@ Draw_Near_Field( void )
 		(fpat.nfeh & NEAR_HFIELD) )
 	{
 	  int ipv; /* Mem request and index */
+	  static size_t mreq = 0;
 
 	  /* Allocate on new near field matrix size */
-	  if( isFlagSet(ALLOC_PNTING_BUFF) )
+	  if( !mreq || isFlagSet(ALLOC_PNTING_BUFF) )
 	  {
-		size_t mreq = (size_t)npts * sizeof( double );
-		mem_realloc( (void *)&pov_x, mreq, "in draw_radiation.c" );
-		mem_realloc( (void *)&pov_y, mreq, "in draw_radiation.c" );
-		mem_realloc( (void *)&pov_z, mreq, "in draw_radiation.c" );
-		mem_realloc( (void *)&pov_r, mreq, "in draw_radiation.c" );
+		mreq = (size_t)npts * sizeof( double );
+		mem_realloc( (void **)&pov_x, mreq, "in draw_radiation.c" );
+		mem_realloc( (void **)&pov_y, mreq, "in draw_radiation.c" );
+		mem_realloc( (void **)&pov_z, mreq, "in draw_radiation.c" );
+		mem_realloc( (void **)&pov_r, mreq, "in draw_radiation.c" );
 		ClearFlag( ALLOC_PNTING_BUFF );
 	  }
 
@@ -546,7 +520,7 @@ Draw_Near_Field( void )
 	  } /* for( ipv = 0; ipv < npts; ipv++ ) */
 
 	  /* Set gc attributes for segment */
-	  Value_to_Color( &rd, &gn, &bl, pov_r[idx], pov_max );
+	  Value_to_Color( &xred, &xgrn, &xblu, pov_r[idx], pov_max );
 
 	  /* Scale factor for each field point, to make
 	   * near field direction lines equal-sized */
@@ -568,7 +542,7 @@ Draw_Near_Field( void )
 		  near_field.pz[idx], fx, fy, fz );
 
 	  /* Draw segment */
-	  cairo_set_source_rgb( cr, rd, gn, bl );
+	  cairo_set_source_rgb( cr, xred, xgrn, xblu );
 	  Cairo_Draw_Line( cr, segm.x1, segm.y1, segm.x2, segm.y2 );
 
 	} /* if( isFlagSet(DRAW_POYNTING) ) */
@@ -577,39 +551,69 @@ Draw_Near_Field( void )
 
   if( isFlagSet(NEAREH_ANIMATE) )
   {
-	cairo_destroy( cr );
 	return;
   }
 
   /* Show max field strength on color code bar */
   if( isFlagSet(DRAW_EFIELD) )
 	max = near_field.max_er;
-  else
-	if( isFlagSet(DRAW_HFIELD) )
-	  max = near_field.max_hr;
-	else
-	  if( isFlagSet(DRAW_POYNTING) )
-		max = pov_max;
+  else if( isFlagSet(DRAW_HFIELD) )
+	max = near_field.max_hr;
+  else if( isFlagSet(DRAW_POYNTING) )
+	max = pov_max;
 
-  snprintf( txt, 9, "%8.2E", max );
-  gtk_label_set_text(
-	  GTK_LABEL(lookup_widget(rdpattern_window,
-		  "rdpattern_colorcode_maxlabel")), txt );
+  snprintf( txt, sizeof(txt), "%8.2E", max );
+  gtk_label_set_text( GTK_LABEL(
+		Builder_Get_Object(rdpattern_window_builder, "rdpattern_colorcode_maxlabel")),
+	  txt );
 
   /* Show min field strength on color code bar */
-  gtk_label_set_text(
-	  GTK_LABEL(lookup_widget(rdpattern_window,
-		  "rdpattern_colorcode_minlabel")), "0" );
+  gtk_label_set_text( GTK_LABEL(
+		Builder_Get_Object(rdpattern_window_builder, "rdpattern_colorcode_minlabel")),
+	  "0" );
 
-  cairo_destroy( cr );
 } /* Draw_Near_Field() */
 
+/*-----------------------------------------------------------------------*/
+
+/* Draw_Radiation()
+ *
+ * Draws the radiation pattern or near E/H fields
+ */
+  void
+Draw_Radiation( cairo_t *cr )
+{
+  /* Abort if xnec2c may be quit by user */
+  if( isFlagSet(MAIN_QUIT) || isFlagClear(ENABLE_EXCITN) )
+	return;
+
+  /* Clear drawingarea */
+  cairo_set_source_rgb( cr, BLACK );
+  cairo_rectangle(
+	  cr, 0.0, 0.0,
+	  (double)rdpattern_proj_params.width,
+	  (double)rdpattern_proj_params.height);
+  cairo_fill( cr );
+
+  /* Draw rad pattern or E/H fields */
+  if( isFlagSet(DRAW_GAIN) )
+	Draw_Radiation_Pattern( cr );
+  else if( isFlagSet(DRAW_EHFIELD) )
+	  Draw_Near_Field( cr );
+
+  /* Display frequency step */
+  Display_Fstep( rdpattern_fstep_entry, calc_data.fstep );
+
+  /* Wait for GTK to complete its tasks */
+  while( g_main_context_iteration(NULL, FALSE) );
+
+} /* Draw_Radiation() */
+
 /*-----------------------------------------------------------------------*/
 
   gboolean
 Animate_Near_Field( gpointer udata )
 {
-  udata = NULL;
   /* omega*t = 2*pi*f*t = Time-varying phase of excitation */
   static double wt = 0.0;
   int idx, npts;
@@ -630,6 +634,7 @@ Animate_Near_Field( gpointer udata )
 		cos( wt + near_field.fey[idx] );
 	  near_field.erz[idx] = near_field.ez[idx] *
 		cos( wt + near_field.fez[idx] );
+
 	  /* Near total electric field vector */
 	  near_field.er[idx]  = sqrt(
 		  near_field.erx[idx] * near_field.erx[idx] +
@@ -648,6 +653,7 @@ Animate_Near_Field( gpointer udata )
 		cos( wt + near_field.fhy[idx] );
 	  near_field.hrz[idx] = near_field.hz[idx] *
 		cos( wt + near_field.fhz[idx] );
+
 	  /* Near total magnetic field vector*/
 	  near_field.hr[idx]  = sqrt(
 		  near_field.hrx[idx] * near_field.hrx[idx] +
@@ -661,10 +667,12 @@ Animate_Near_Field( gpointer udata )
 
   /* Increment excitation phase, keep < 2pi */
   wt += near_field.anim_step;
-  if( wt >= (double)TP )
+  if( wt >= (double)M_2PI )
 	wt = 0.0;
 
-  Draw_Radiation( rdpattern_drawingarea );
+  /* Wait for GTK to complete its tasks */
+  gtk_widget_queue_draw( rdpattern_drawingarea );
+  while( g_main_context_iteration(NULL, FALSE) );
 
   return( TRUE );
 
@@ -672,62 +680,15 @@ Animate_Near_Field( gpointer udata )
 
 /*-----------------------------------------------------------------------*/
 
-/* Scale_Gain()
- *
- * Scales radiation pattern gain according to selected style
- * ( ARRL style, logarithmic or linear voltage/power )
- */
-static int gain_style = GS_LINP;
-
-  double
-Scale_Gain( double gain, int fstep, int idx )
-{
-  /* Scaled rad pattern gain and pol factor */
-  double rad = 0.0, polf;
-
-  polf = Polarization_Factor( calc_data.pol_type, fstep, idx );
-  if ( polf< 1e-200 ) return( 0.0 );
-
-  gain += 10.0 * log10(polf);
-
-  switch( gain_style )
-  {
-	case GS_LINP:
-	  rad = pow(10.0, (gain/10.0));
-	  break;
-
-	case GS_LINV:
-	  rad = pow(10.0, (gain/20.0));
-	  break;
-
-	case GS_ARRL:
-	  rad = exp( 0.058267 * gain );
-	  break;
-
-	case GS_LOG:
-	  rad = gain;
-	  if( rad < -40 )
-		rad = 0.0;
-	  else
-		rad = rad /40.0 + 1.0;
-
-  } /* switch( gain_style ) */
-
-  return( rad );
-
-} /* Scale_Gain() */
-
-/*-----------------------------------------------------------------------*/
-
 /* Polarization_Factor()
  *
  * Calculates polarization factor from axial
- * ratio and tilt of polarization ellepse
+ * ratio and tilt of polarization ellipse
  */
   double
 Polarization_Factor( int pol_type, int fstep, int idx )
 {
-  double axrt2, tlt, polf = 1.0;
+  double axrt, axrt2, tilt2, polf = 1.0;
 
   switch( pol_type )
   {
@@ -736,37 +697,37 @@ Polarization_Factor( int pol_type, int fstep, int idx )
 	  break;
 
 	case POL_HORIZ:
-	  axrt2 = rad_pattern[fstep].axrt[idx] *
-		rad_pattern[fstep].axrt[idx];
-	  tlt = sin( rad_pattern[fstep].tilt[idx] );
-	  polf = (axrt2 + (1.0 - axrt2) * tlt * tlt) /
-		(1.0 + axrt2);
+	  axrt2  = rad_pattern[fstep].axrt[idx];
+	  axrt2 *= axrt2;
+	  tilt2  = sin( rad_pattern[fstep].tilt[idx] );
+	  tilt2 *= tilt2;
+	  polf = (axrt2 + (1.0 - axrt2) * tilt2) / (1.0 + axrt2);
 	  break;
 
 	case POL_VERT:
-	  axrt2 = rad_pattern[fstep].axrt[idx] *
-		rad_pattern[fstep].axrt[idx];
-	  tlt = cos( rad_pattern[fstep].tilt[idx] );
-	  polf = (axrt2 + (1.0 - axrt2) * tlt * tlt) /
-		(1.0 + axrt2);
+	  axrt2  = rad_pattern[fstep].axrt[idx];
+	  axrt2 *= axrt2;
+	  tilt2  = cos( rad_pattern[fstep].tilt[idx] );
+	  tilt2 *= tilt2;
+	  polf = (axrt2 + (1.0 - axrt2) * tilt2) / (1.0 + axrt2);
 	  break;
 
 	case POL_LHCP:
-	  axrt2 = rad_pattern[fstep].axrt[idx] *
-		rad_pattern[fstep].axrt[idx];
-	  polf = (1.0 + 2.0 * rad_pattern[fstep].axrt[idx] +
-		  axrt2) / 2.0 / (1.0 + axrt2);
+	  axrt  = rad_pattern[fstep].axrt[idx];
+	  axrt2 = axrt * axrt;
+	  polf  = (1.0 + 2.0 * axrt + axrt2) / 2.0 / (1.0 + axrt2);
 	  break;
 
 	case POL_RHCP:
-	  axrt2 = rad_pattern[fstep].axrt[idx] *
-		rad_pattern[fstep].axrt[idx];
-	  polf = (1.0 - 2.0 * rad_pattern[fstep].axrt[idx] +
-		  axrt2) / 2.0 / (1.0 + axrt2);
+	  axrt  = rad_pattern[fstep].axrt[idx];
+	  axrt2 = axrt * axrt;
+	  polf  = (1.0 - 2.0 * axrt + axrt2) / 2.0 / (1.0 + axrt2);
   }
 
-  return( polf );
+  if( polf < 1.0E-200 ) polf = 1.0E-200;
+  polf = 10.0 * log10( polf );
 
+  return( polf );
 } /* Polarization_Factor() */
 
 /*-----------------------------------------------------------------------*/
@@ -783,20 +744,30 @@ Set_Polarization( int pol )
 
   Set_Window_Labels();
 
-  /* Trigger a redraw of drawingareas */
-  if( isFlagSet(DRAW_ENABLED) )
-	Draw_Radiation( rdpattern_drawingarea );
-
-  if( isFlagSet(PLOT_ENABLED) )
-	Plot_Frequency_Data();
-
   /* Show gain in direction of viewer */
-  Show_Viewer_Gain( main_window,
-	  "main_gain_entry", structure_proj_params );
+  Show_Viewer_Gain(
+	  main_window_builder,
+	  "main_gain_entry",
+	  structure_proj_params );
 
   /* Enable redraw of rad pattern */
   SetFlag( DRAW_NEW_RDPAT );
 
+  /* Trigger a redraw of drawingareas */
+  if( isFlagSet(DRAW_ENABLED) )
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( rdpattern_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
+
+  if( isFlagSet(PLOT_ENABLED) )
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( freqplots_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
+
 } /* Set_Polarization() */
 
 /*-----------------------------------------------------------------------*/
@@ -817,7 +788,10 @@ Set_Gain_Style( int gs )
   {
 	/* Enable redraw of rad pattern */
 	SetFlag( DRAW_NEW_RDPAT );
-	Draw_Radiation( rdpattern_drawingarea );
+
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( rdpattern_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
   }
 
 } /* Set_Gain_Style() */
@@ -833,18 +807,26 @@ Set_Gain_Style( int gs )
 New_Radiation_Projection_Angle(void)
 {
   /* sin and cos of rad pattern rotation and inclination angles */
-  rdpattern_proj_params.sin_wr = sin(rdpattern_proj_params.Wr/(double)TD);
-  rdpattern_proj_params.cos_wr = cos(rdpattern_proj_params.Wr/(double)TD);
-  rdpattern_proj_params.sin_wi = sin(rdpattern_proj_params.Wi/(double)TD);
-  rdpattern_proj_params.cos_wi = cos(rdpattern_proj_params.Wi/(double)TD);
+  rdpattern_proj_params.sin_wr = sin(rdpattern_proj_params.Wr/(double)TODEG);
+  rdpattern_proj_params.cos_wr = cos(rdpattern_proj_params.Wr/(double)TODEG);
+  rdpattern_proj_params.sin_wi = sin(rdpattern_proj_params.Wi/(double)TODEG);
+  rdpattern_proj_params.cos_wi = cos(rdpattern_proj_params.Wi/(double)TODEG);
 
   /* Trigger a redraw of radiation drawingarea */
   if( isFlagSet(DRAW_ENABLED) )
-	Draw_Radiation( rdpattern_drawingarea );
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( rdpattern_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
   /* Trigger a redraw of plots drawingarea if doing "viewer" gain */
   if( isFlagSet(PLOT_ENABLED) && isFlagSet(PLOT_GVIEWER) )
-	Plot_Frequency_Data();
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( freqplots_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
 } /* New_Radiation_Projection_Angle() */
 
@@ -857,7 +839,6 @@ New_Radiation_Projection_Angle(void)
   gboolean
 Redo_Radiation_Pattern( gpointer udata )
 {
-  udata = NULL;
   /* Redo radiation pattern for a new frequency. Below
    * makes calcs use the extra buffer in rad_pattern */
   calc_data.fstep = calc_data.nfrq;
@@ -865,7 +846,11 @@ Redo_Radiation_Pattern( gpointer udata )
 
   /* Redraw radiation pattern on screen */
   if( isFlagSet(DRAW_ENABLED) )
-	Draw_Radiation( rdpattern_drawingarea );
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( rdpattern_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
   return FALSE;
 
@@ -881,39 +866,46 @@ Redo_Radiation_Pattern( gpointer udata )
   double
 Viewer_Gain( projection_parameters_t proj_parameters, int fstep )
 {
-  double phi, gain=-999;
+  double phi, gain;
   int nth, nph, idx;
 
-
   /* Calculate theta step from proj params */
   phi = proj_parameters.Wr;
-  if( fpat.dth == 0.0 )	nth = 0;
+  if( fpat.dth == 0.0 ) nth = 0;
   else
   {
-	double theta = fabs( 90.0 - proj_parameters.Wi );
+	double theta;
+	theta = fabs( 90.0 - proj_parameters.Wi );
 	if( theta > 180.0 )
 	{
 	  theta = 360.0 - theta;
 	  phi  -= 180.0;
 	}
-	if( (gnd.ksymp == 2) && (theta > 90.01) && (gnd.ifar != 1) )
-	  return( -999.0 );
-	nth = (int)( theta/fpat.dth + 0.5 );
-	if( (nth >= fpat.nth) || (nth < 0) ) nth = fpat.nth-1;
+
+	if( (gnd.ksymp == 2) &&
+		(theta > 90.01)  &&
+		(gnd.ifar != 1) )
+	  return( -999.99 );
+
+	nth = (int)( (theta - fpat.thets) / fpat.dth + 0.5 );
+	if( (nth >= fpat.nth) || (nth < 0) )
+	  nth = fpat.nth-1;
   }
 
   /* Calculate phi step from proj params */
-  if( fpat.dph == 0.0 )	nph = 0;
+  if( fpat.dph == 0.0 ) nph = 0;
   else
   {
 	while( phi < 0.0 ) phi += 360.0;
-	nph = (int)( phi/fpat.dph + 0.5 );
-	if( (nph >= fpat.nph) || (nph < 0) ) nph = fpat.nph-1;
+	nph = (int)( (phi - fpat.phis) / fpat.dph + 0.5 );
+	if( (nph >= fpat.nph) || (nph < 0) )
+	  nph = fpat.nph-1;
   }
 
-  idx = nth + nph*fpat.nth;
-  gain = rad_pattern[fstep].gtot[idx] +	10.0 *
-	log10( Polarization_Factor(calc_data.pol_type, fstep, idx) );
+  idx = nth + nph * fpat.nth;
+  gain = rad_pattern[fstep].gtot[idx] +
+		Polarization_Factor(calc_data.pol_type, fstep, idx);
+  if( gain < -999.99 ) gain = -999.99;
 
   return( gain );
 
@@ -933,7 +925,7 @@ Rdpattern_Window_Killed( void )
 	gtk_widget_destroy( animate_dialog );
 	animate_dialog = NULL;
 	ClearFlag( NEAREH_ANIMATE );
-	g_source_remove( anim_tag );
+	if( anim_tag ) g_source_remove( anim_tag );
 	anim_tag = 0;
   }
 
@@ -942,11 +934,13 @@ Rdpattern_Window_Killed( void )
 	ClearFlag( DRAW_FLAGS );
 	rdpattern_drawingarea = NULL;
 	rdpattern_window = NULL;
+	g_object_unref( rdpattern_window_builder );
+	rdpattern_window_builder = NULL;
 	Free_Draw_Buffers();
 
-	gtk_check_menu_item_set_active(
-		GTK_CHECK_MENU_ITEM(lookup_widget(
-			main_window, "main_rdpattern")), FALSE );
+	gtk_check_menu_item_set_active(	GTK_CHECK_MENU_ITEM(
+		  Builder_Get_Object( main_window_builder, "main_rdpattern")),
+		FALSE );
   }
 
 } /* Rdpattern_Window_Killed() */
@@ -979,41 +973,41 @@ Set_Window_Labels( void )
   };
 
   char txt[64];
+  size_t s = sizeof( txt );
 
   if( isFlagSet(DRAW_ENABLED) )
   {
 	/* Set window labels */
-	strcpy( txt, _("Radiation Patterns") );
+	Strlcpy( txt, _("Radiation Patterns"), s );
 	if( isFlagSet(DRAW_GAIN) )
 	{
-	  strcpy( txt, _("Radiation Pattern: - ") );
-	  strcat( txt, pol_type[calc_data.pol_type] );
-	  strcat( txt, " - " );
-	  strcat( txt, scale[gain_style] );
+	  Strlcpy( txt, _("Radiation Pattern: - "), s );
+	  Strlcat( txt, pol_type[calc_data.pol_type], s );
+	  Strlcat( txt, " - ", s );
+	  Strlcat( txt, scale[gain_style], s );
+	}
+	else if( isFlagSet(DRAW_EHFIELD) )
+	{
+	  Strlcpy( txt, _("Near Fields:"), s );
+	  if( isFlagSet(DRAW_EFIELD) )
+		Strlcat( txt, _(" - E Field"), s );
+	  if( isFlagSet(DRAW_HFIELD) )
+		Strlcat( txt, _(" - H Field"), s );
+	  if( isFlagSet(DRAW_POYNTING) )
+		Strlcat( txt, _(" - Poynting Vector"), s );
 	}
-	else
-	  if( isFlagSet(DRAW_EHFIELD) )
-	  {
-		strcpy( txt, _("Near Fields:") );
-		if( isFlagSet(DRAW_EFIELD) )
-		  strcat( txt, _(" - E Field") );
-		if( isFlagSet(DRAW_HFIELD) )
-		  strcat( txt, _(" - H Field") );
-		if( isFlagSet(DRAW_POYNTING) )
-		  strcat( txt, _(" - Poynting Vector") );
-	  }
 
-	gtk_label_set_text( GTK_LABEL(lookup_widget(
-			rdpattern_window, "rdpattern_label")), txt );
+	gtk_label_set_text( GTK_LABEL(
+		  Builder_Get_Object(rdpattern_window_builder, "rdpattern_label")), txt );
 
   } /* if( isFlagSet(DRAW_ENABLED) ) */
 
   if( isFlagSet(PLOT_ENABLED) )
   {
-	strcpy( txt, _("Frequency Data Plots - ") );
-	strcat( txt, pol_type[calc_data.pol_type] );
-	gtk_label_set_text( GTK_LABEL(lookup_widget(
-			freqplots_window, "freqplots_label")), txt );
+	Strlcpy( txt, _("Frequency Data Plots - "), s );
+	Strlcat( txt, pol_type[calc_data.pol_type], s );
+	gtk_label_set_text( GTK_LABEL(
+		  Builder_Get_Object(freqplots_window_builder, "freqplots_label")), txt );
   }
 
 } /* Set_Window_Labels() */
@@ -1034,53 +1028,52 @@ Alloc_Rdpattern_Buffers( int nfrq, int nth, int nph )
   /* Free old gain buffers first */
   for( idx = 0; idx < last_nfrq; idx++ )
   {
-	free_ptr( (void *)&rad_pattern[idx].gtot );
-	free_ptr( (void *)&rad_pattern[idx].max_gain );
-	free_ptr( (void *)&rad_pattern[idx].min_gain );
-	free_ptr( (void *)&rad_pattern[idx].max_gain_tht );
-	free_ptr( (void *)&rad_pattern[idx].max_gain_phi );
-	free_ptr( (void *)&rad_pattern[idx].max_gain_idx );
-	free_ptr( (void *)&rad_pattern[idx].min_gain_idx );
-	free_ptr( (void *)&rad_pattern[idx].axrt );
-	free_ptr( (void *)&rad_pattern[idx].tilt );
-	free_ptr( (void *)&rad_pattern[idx].sens );
+	free_ptr( (void **)&rad_pattern[idx].gtot );
+	free_ptr( (void **)&rad_pattern[idx].max_gain );
+	free_ptr( (void **)&rad_pattern[idx].min_gain );
+	free_ptr( (void **)&rad_pattern[idx].max_gain_tht );
+	free_ptr( (void **)&rad_pattern[idx].max_gain_phi );
+	free_ptr( (void **)&rad_pattern[idx].max_gain_idx );
+	free_ptr( (void **)&rad_pattern[idx].min_gain_idx );
+	free_ptr( (void **)&rad_pattern[idx].axrt );
+	free_ptr( (void **)&rad_pattern[idx].tilt );
+	free_ptr( (void **)&rad_pattern[idx].sens );
   }
   last_nfrq = nfrq;
 
   /* Allocate rad pattern buffers */
   mreq = (size_t)nfrq * sizeof(rad_pattern_t);
-  mem_realloc( (void *)&rad_pattern, mreq, "in draw_radiation.c" );
-
+  mem_realloc( (void **)&rad_pattern, mreq, "in draw_radiation.c" );
   for( idx = 0; idx < nfrq; idx++ )
   {
 	/* Memory request for allocs */
 	mreq = (size_t)(nph * nth) * sizeof(double);
 	rad_pattern[idx].gtot = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].gtot, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].gtot), mreq, "in draw_radiation.c" );
 	rad_pattern[idx].axrt = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].axrt, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].axrt), mreq, "in draw_radiation.c" );
 	rad_pattern[idx].tilt = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].tilt, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].tilt), mreq, "in draw_radiation.c" );
 
 	mreq = NUM_POL * sizeof(double);
 	rad_pattern[idx].max_gain = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].max_gain, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].max_gain), mreq, "in draw_radiation.c" );
 	rad_pattern[idx].min_gain = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].min_gain, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].min_gain), mreq, "in draw_radiation.c" );
 	rad_pattern[idx].max_gain_tht = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].max_gain_tht, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].max_gain_tht), mreq, "in draw_radiation.c" );
 	rad_pattern[idx].max_gain_phi = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].max_gain_phi, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].max_gain_phi), mreq, "in draw_radiation.c" );
 
 	mreq = NUM_POL * sizeof(int);
 	rad_pattern[idx].max_gain_idx = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].max_gain_idx, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].max_gain_idx), mreq, "in draw_radiation.c" );
 	rad_pattern[idx].min_gain_idx = NULL;
-	mem_alloc( (void *)&rad_pattern[idx].min_gain_idx, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].min_gain_idx), mreq, "in draw_radiation.c" );
 
 	rad_pattern[idx].sens = NULL;
 	mreq = (size_t)(nph * nth) * sizeof(int);
-	mem_alloc( (void *)&rad_pattern[idx].sens, mreq, "in draw_radiation.c" );
+	mem_alloc( (void **)&(rad_pattern[idx].sens), mreq, "in draw_radiation.c" );
   }
 
 } /* Alloc_Rdpattern_Buffers() */
@@ -1105,35 +1098,35 @@ Alloc_Nearfield_Buffers( int n1, int n2, int n3 )
   /* Allocate near field buffers */
   if( fpat.nfeh & NEAR_EFIELD )
   {
-	mem_realloc( (void *)&near_field.ex,  mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.ey,  mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.ez,  mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.fex, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.fey, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.fez, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.erx, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.ery, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.erz, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.er,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.ex,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.ey,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.ez,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.fex, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.fey, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.fez, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.erx, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.ery, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.erz, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.er,  mreq, "in draw_radiation.c" );
   }
 
   if( fpat.nfeh & NEAR_HFIELD )
   {
-	mem_realloc( (void *)&near_field.hx,  mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.hy,  mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.hz,  mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.fhx, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.fhy, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.fhz, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.hrx, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.hry, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.hrz, mreq, "in draw_radiation.c" );
-	mem_realloc( (void *)&near_field.hr,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hx,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hy,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hz,  mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.fhx, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.fhy, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.fhz, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hrx, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hry, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hrz, mreq, "in draw_radiation.c" );
+	mem_realloc( (void **)&near_field.hr,  mreq, "in draw_radiation.c" );
   }
 
-  mem_realloc( (void *)&near_field.px, mreq, "in draw_radiation.c" );
-  mem_realloc( (void *)&near_field.py, mreq, "in draw_radiation.c" );
-  mem_realloc( (void *)&near_field.pz, mreq, "in draw_radiation.c" );
+  mem_realloc( (void **)&near_field.px, mreq, "in draw_radiation.c" );
+  mem_realloc( (void **)&near_field.py, mreq, "in draw_radiation.c" );
+  mem_realloc( (void **)&near_field.pz, mreq, "in draw_radiation.c" );
 
 } /* Alloc_Nearfield_Buffers() */
 
@@ -1146,10 +1139,10 @@ Alloc_Nearfield_Buffers( int n1, int n2, int n3 )
   void
 Free_Draw_Buffers( void )
 {
-  free_ptr( (void *)&point_3d );
-  free_ptr( (void *)&red );
-  free_ptr( (void *)&grn );
-  free_ptr( (void *)&blu );
+  free_ptr( (void **)&point_3d );
+  free_ptr( (void **)&red );
+  free_ptr( (void **)&grn );
+  free_ptr( (void **)&blu );
 }
 
 /*-----------------------------------------------------------------------*/
