@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -17,12 +14,6 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/*
- *  draw_structure.c
- *
- *  Structure drawing routines for xnec2c
- */
-
 #include "draw_structure.h"
 #include "shared.h"
 
@@ -33,39 +24,31 @@
  *  Draws xyz axes, wire segments and patches
  */
   void
-Draw_Structure( GtkWidget *drawingarea )
+Draw_Structure( cairo_t *cr )
 {
-  /* Block motion events */
-  g_signal_handler_block( (gpointer)drawingarea, structure_motion_handler );
-
   /* Abort if xnec2c may be quit by user */
-  if( isFlagSet(MAIN_QUIT) )
-	return;
+  if( isFlagSet(MAIN_QUIT) ) return;
 
-  /* Cairo context */
-  cairo_t *cr = gdk_cairo_create( structure_pixmap );
-
-  /* Clear pixmap */
+  /* Clear drawingarea */
   cairo_set_source_rgb( cr, BLACK );
   cairo_rectangle(
 	  cr, 0.0, 0.0,
-	  (double)structure_proj_params.pixmap_width,
-	  (double)structure_proj_params.pixmap_height);
+	  (double)structure_proj_params.width,
+	  (double)structure_proj_params.height);
   cairo_fill( cr );
 
   /* Process and draw geometry if available, else clear screen */
   Process_Wire_Segments();
   Process_Surface_Patches();
-  Draw_XYZ_Axes( structure_pixmap, structure_proj_params );
-  Draw_Surface_Patches( structure_segs+data.n, data.m );
-  Draw_Wire_Segments( structure_segs, data.n );
+  Draw_XYZ_Axes( cr, structure_proj_params );
+  Draw_Surface_Patches( cr, structure_segs+data.n, data.m );
+  Draw_Wire_Segments( cr, structure_segs, data.n );
 
   /* Show gain in direction of viewer */
-  Show_Viewer_Gain( main_window, "main_gain_entry", structure_proj_params );
-
-  /* Render pixmap to screen */
-  gdk_window_set_back_pixmap( drawingarea->window, structure_pixmap, FALSE );
-  gdk_window_clear( drawingarea->window );
+  Show_Viewer_Gain(
+	  main_window_builder,
+	  "main_gain_entry",
+	  structure_proj_params );
 
   /* Reset "new current data" flag */
   crnt.newer = 0;
@@ -76,11 +59,6 @@ Draw_Structure( GtkWidget *drawingarea )
   /* Wait for GTK to complete its tasks */
   while( g_main_context_iteration(NULL, FALSE) );
 
-  cairo_destroy( cr );
-
-  /* Unblock motion events */
-  g_signal_handler_unblock( (gpointer)drawingarea, structure_motion_handler );
-
 } /* Draw_Structure() */
 
 /*-----------------------------------------------------------------------*/
@@ -90,7 +68,7 @@ Draw_Structure( GtkWidget *drawingarea )
  *  Calculates some projection parameters
  *  when new wire segment data is created
  */
-  void
+  static void
 New_Wire_Data( void )
 {
   /* Abort if no wire data */
@@ -128,8 +106,8 @@ New_Wire_Data( void )
 
   /* Redraw structure on screen */
   New_Projection_Parameters(
-	  structure_pixmap_width,
-	  structure_pixmap_height,
+	  structure_width,
+	  structure_height,
 	  &structure_proj_params );
 
 } /* New_Wire_Data() */
@@ -158,12 +136,12 @@ New_Patch_Data( void )
 
   /* Allocate memory for patch line segments */
   mreq = (size_t)(2 * data.m) * sizeof(double);
-  mem_realloc( (void *)&data.px1, mreq, "in draw_structure.c" );
-  mem_realloc( (void *)&data.py1, mreq, "in draw_structure.c" );
-  mem_realloc( (void *)&data.pz1, mreq, "in draw_structure.c" );
-  mem_realloc( (void *)&data.px2, mreq, "in draw_structure.c" );
-  mem_realloc( (void *)&data.py2, mreq, "in draw_structure.c" );
-  mem_realloc( (void *)&data.pz2, mreq, "in draw_structure.c" );
+  mem_realloc( (void **)&data.px1, mreq, "in draw_structure.c" );
+  mem_realloc( (void **)&data.py1, mreq, "in draw_structure.c" );
+  mem_realloc( (void **)&data.pz1, mreq, "in draw_structure.c" );
+  mem_realloc( (void **)&data.px2, mreq, "in draw_structure.c" );
+  mem_realloc( (void **)&data.py2, mreq, "in draw_structure.c" );
+  mem_realloc( (void **)&data.pz2, mreq, "in draw_structure.c" );
 
   /* Find point furthest from xyz axes origin */
   r_max = 0.0;
@@ -246,8 +224,8 @@ New_Patch_Data( void )
 
   /* Redraw structure on screen */
   New_Projection_Parameters(
-	  structure_pixmap_width,
-	  structure_pixmap_height,
+	  structure_width,
+	  structure_height,
 	  &structure_proj_params );
 
 } /* New_Patch_Data() */
@@ -312,8 +290,8 @@ Process_Surface_Patches( void )
  *  Draws all wire segments of the input structure
  */
 
-void
-Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
+  void
+Draw_Wire_Segments(	cairo_t *cr, Segment_t *segm, gint nseg )
 {
   /* Abort if no wire segs or new input pending */
   if( !nseg || isFlagSet(INPUT_PENDING) )
@@ -321,9 +299,6 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
 
   int idx, i;
 
-  /* Cairo context */
-  cairo_t *cr = gdk_cairo_create( structure_pixmap );
-
   /* Draw networks */
   for( idx = 0; idx < netcx.nonet; idx++ )
   {
@@ -352,7 +327,7 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
 		  Cairo_Draw_Polygon( cr, points, 4 );
 		  cairo_fill( cr );
 
-			/* Draw connecting lines */
+		  /* Draw connecting lines */
 		  Cairo_Draw_Line( cr,
 			  segm[i1].x1, segm[i1].y1,
 			  segm[i2].x1, segm[i2].y1 );
@@ -398,8 +373,10 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
 	  crnt.valid )
   {
 	static double cmax; /* Max of seg current/charge */
-	double red, grn, blu;  /* To color structure segs   */
+	/* To color structure segs */
+	double red = 0.0, grn = 0.0, blu = 0.0;
 	char label[11];
+	size_t s = sizeof( label );
 
 	/* Loop over all wire segs, find max current/charge */
 	if( crnt.newer )
@@ -421,12 +398,12 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
 
 	  /* Show max value in color code label */
 	  if( isFlagSet(DRAW_CURRENTS) )
-		snprintf( label, 11, "%10.3E", cmax * (double)data.wlam );
+		snprintf( label, s, "%10.3E", cmax * (double)data.wlam );
 	  else
-		snprintf( label, 11, "%10.3E", cmax * 1.0E-6/(double)calc_data.fmhz );
-	  gtk_label_set_text(
-		  GTK_LABEL(lookup_widget(structure_drawingarea,
-			  "main_colorcode_maxlabel")), label );
+		snprintf( label, s, "%10.3E", cmax * 1.0E-6/(double)calc_data.fmhz );
+	  gtk_label_set_text( GTK_LABEL(
+			Builder_Get_Object(main_window_builder, "main_colorcode_maxlabel")),
+		  label );
 
 	} /* if( crnt.newer ) */
 
@@ -445,7 +422,6 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
 		  segm[idx].x2, segm[idx].y2 );
 	} /* for( idx = 0; idx < nseg; idx++ ) */
 
-	cairo_destroy( cr );
 	return;
   } /* if( isFlagSet(DRAW_CURRENTS) || isFlagSet(DRAW_CHARGES) ) */
 
@@ -513,7 +489,6 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
 	}
   }
 
-  cairo_destroy( cr );
 } /* Draw_Wire_Segments() */
 
 /*-----------------------------------------------------------------------*/
@@ -522,15 +497,11 @@ Draw_Wire_Segments(	GdkSegment *segm, gint nseg )
  *
  *  Draws the line segments that represent surface patches
  */
-void
-Draw_Surface_Patches( GdkSegment *segm, gint npatch )
+  void
+Draw_Surface_Patches( cairo_t *cr, Segment_t *segm, gint npatch )
 {
   /* Abort if no patches */
-  if( ! npatch )
-	return;
-
-  /* Cairo context */
-  cairo_t *cr = gdk_cairo_create( structure_pixmap );
+  if( ! npatch ) return;
 
   /* Draw currents if enabled, return */
   if( isFlagSet(DRAW_CURRENTS) && crnt.valid )
@@ -541,7 +512,7 @@ Draw_Surface_Patches( GdkSegment *segm, gint npatch )
 	/* Current along x,y,z and t1,t2 vector directions */
 	complex double cx, cy, cz, ct1, ct2;
 
-	double red, grn, blu;
+	double red = 0.0, grn = 0.0, blu = 0.0;
 
 	int i, j;
 
@@ -554,9 +525,9 @@ Draw_Surface_Patches( GdkSegment *segm, gint npatch )
 	  for( i = 0; i < npatch; i++ )
 	  {
 		/* Calculate current along x,y,z vectors */
-		cx = (complex double)crnt.cur[j];
-		cy = (complex double)crnt.cur[j+1];
-		cz = (complex double)crnt.cur[j+2];
+		cx = crnt.cur[j];
+		cy = crnt.cur[j+1];
+		cz = crnt.cur[j+2];
 
 		/* Calculate current along t1 and t2 tangent vectors */
 		ct1 = cx*(double)data.t1x[i] +
@@ -619,8 +590,7 @@ Draw_Surface_Patches( GdkSegment *segm, gint npatch )
 	if( isFlagSet(OVERLAY_STRUCT) &&
 		(structure_proj_params.type == RDPATTERN_DRAWINGAREA) )
 	  cairo_set_source_rgb( cr, WHITE );
-	else
-	  cairo_set_source_rgb( cr, BLUE );
+	else cairo_set_source_rgb( cr, BLUE );
 
 	/* Draw patch segments */
 	int nsg = 2 * npatch;
@@ -632,7 +602,6 @@ Draw_Surface_Patches( GdkSegment *segm, gint npatch )
 	}
   }
 
-  cairo_destroy( cr );
 } /* Draw_Surface_Patches() */
 
 /*-----------------------------------------------------------------------*/
@@ -644,7 +613,6 @@ Draw_Surface_Patches( GdkSegment *segm, gint npatch )
   gboolean
 Redo_Currents( gpointer udata )
 {
-  udata = NULL;
   /* Abort if no geometry data */
   if( ((data.n == 0) && (data.m == 0)) ||
 	  isFlagClear(ENABLE_EXCITN) )
@@ -657,12 +625,20 @@ Redo_Currents( gpointer udata )
 
   /* Display freq data in entry widgets */
   if( isFlagSet(PLOT_FREQ_LINE) )
-	Plot_Frequency_Data();
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( freqplots_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
   /* Redraw structure on screen */
   if( (structure_drawingarea != NULL) &&
 	  (isFlagSet(DRAW_CURRENTS) || isFlagSet(DRAW_CHARGES)) )
-	Draw_Structure( structure_drawingarea );
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( structure_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
   return FALSE;
 } /* Redo_Currents() */
@@ -678,17 +654,26 @@ Redo_Currents( gpointer udata )
 New_Structure_Projection_Angle(void)
 {
   /* sin and cos of structure rotation and inclination angles */
-  structure_proj_params.sin_wr = sin(structure_proj_params.Wr/(double)TD);
-  structure_proj_params.cos_wr = cos(structure_proj_params.Wr/(double)TD);
-  structure_proj_params.sin_wi = sin(structure_proj_params.Wi/(double)TD);
-  structure_proj_params.cos_wi = cos(structure_proj_params.Wi/(double)TD);
+  structure_proj_params.sin_wr = sin(structure_proj_params.Wr/(double)TODEG);
+  structure_proj_params.cos_wr = cos(structure_proj_params.Wr/(double)TODEG);
+  structure_proj_params.sin_wi = sin(structure_proj_params.Wi/(double)TODEG);
+  structure_proj_params.cos_wi = cos(structure_proj_params.Wi/(double)TODEG);
 
   /* Trigger a redraw of structure drawingarea */
-  Draw_Structure( structure_drawingarea );
+  if( isFlagClear(INPUT_PENDING) )
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( structure_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
   /* Trigger a redraw of plots drawingarea */
   if( isFlagSet(PLOT_ENABLED) && isFlagSet(PLOT_GVIEWER) )
-	Plot_Frequency_Data();
+  {
+	/* Wait for GTK to complete its tasks */
+	gtk_widget_queue_draw( freqplots_drawingarea );
+	while( g_main_context_iteration(NULL, FALSE) );
+  }
 
 } /* New_Structure_Projection_Angle() */
 
@@ -702,8 +687,8 @@ New_Structure_Projection_Angle(void)
 Init_Struct_Drawing( void )
 {
   /* We need n segs for wires + 2m for patces */
-  size_t mreq = (size_t)(data.n + 2*data.m) * sizeof(GdkSegment);
-  mem_realloc( (void *)&structure_segs, mreq, "in draw_structure.c" );
+  size_t mreq = (size_t)(data.n + 2*data.m) * sizeof(Segment_t);
+  mem_realloc( (void **)&structure_segs, mreq, "in draw_structure.c" );
   New_Wire_Data();
   New_Patch_Data();
 }
@@ -716,8 +701,8 @@ Init_Struct_Drawing( void )
  */
   void
 Show_Viewer_Gain(
-	GtkWidget *window,
-	const char *widget,
+	GtkBuilder *builder,
+	gchar *widget,
 	projection_parameters_t proj_params )
 {
   if( isFlagSet(DRAW_CURRENTS)	||
@@ -727,12 +712,12 @@ Show_Viewer_Gain(
   {
 	char txt[8];
 
-	if( isFlagSet(ENABLE_RDPAT) && (calc_data.fstep >=0) )
+	if( isFlagSet(ENABLE_RDPAT) && (calc_data.fstep >= 0) )
 	{
-	  snprintf( txt, 8, "%7.2f",
+	  snprintf( txt, sizeof(txt), "%7.2f",
 		  Viewer_Gain(proj_params, calc_data.fstep) );
-	  txt[7] = '\0';
-	  gtk_entry_set_text( GTK_ENTRY(lookup_widget(window, widget)), txt );
+	  gtk_entry_set_text(GTK_ENTRY(
+			Builder_Get_Object(builder, widget)), txt );
 	}
   }
 
