@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -76,20 +73,20 @@ arc( int itg, int ns, double rada,
 
 	/* Reallocate tags buffer */
 	mreq = (size_t)(data.n + data.m) * sizeof(int);
-	mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
 	/* Reallocate wire buffers */
 	mreq = (size_t)data.n * sizeof(double);
-	mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
-	ang= ang1* TA;
-	dang=( ang2- ang1)* TA/ ns;
+	ang= ang1* TORAD;
+	dang=( ang2- ang1)* TORAD/ ns;
 	xs1= rada* cos( ang);
 	zs1= rada* sin( ang);
 
@@ -126,6 +123,121 @@ arc( int itg, int ns, double rada,
 
 /*-----------------------------------------------------------------------*/
 
+/*** this function was an 'entry point' (part of) 'patch()' ***/
+  static void
+subph( int nx, int ny )
+{
+  int mia, ix, iy, mi;
+  size_t mreq;
+  double xs, ys, zs, xa, xst, s1x, s1y;
+  double s1z, s2x, s2y, s2z, saln, xt, yt;
+
+  /* Reallocate patch buffers */
+  if( ny == 0 ) data.m += 3;
+  else data.m += 4;
+
+  mreq = (size_t)data.m * sizeof(double);
+  mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
+  if( !CHILD )
+  {
+	mreq = (size_t)(data.n + 2 * data.m) * sizeof(Segment_t);
+	mem_realloc( (void **)&structure_segs, mreq, "in geometry.c" );
+  }
+
+  /* Shift patches to make room for new ones */
+  if( (ny == 0) && (nx != data.m) )
+  {
+	for( iy = data.m-1; iy > nx+2; iy-- )
+	{
+	  ix = iy-3;
+	  data.px[iy]= data.px[ix];
+	  data.py[iy]= data.py[ix];
+	  data.pz[iy]= data.pz[ix];
+	  data.pbi[iy]= data.pbi[ix];
+	  data.psalp[iy]= data.psalp[ix];
+	  data.t1x[iy]= data.t1x[ix];
+	  data.t1y[iy]= data.t1y[ix];
+	  data.t1z[iy]= data.t1z[ix];
+	  data.t2x[iy]= data.t2x[ix];
+	  data.t2y[iy]= data.t2y[ix];
+	  data.t2z[iy]= data.t2z[ix];
+	}
+
+  } /* if( (ny == 0) || (nx != m) ) */
+
+  /* divide patch for connection */
+  mi= nx-1;
+  xs= data.px[mi];
+  ys= data.py[mi];
+  zs= data.pz[mi];
+  xa= data.pbi[mi]/4.0;
+  xst= sqrt( xa)/2.0;
+  s1x= data.t1x[mi];
+  s1y= data.t1y[mi];
+  s1z= data.t1z[mi];
+  s2x= data.t2x[mi];
+  s2y= data.t2y[mi];
+  s2z= data.t2z[mi];
+  saln= data.psalp[mi];
+  xt= xst;
+  yt= xst;
+
+  if( ny == 0)
+	mia= mi;
+  else
+  {
+	data.mp++;
+	mia= data.m-1;
+  }
+
+  for( ix = 1; ix <= 4; ix++ )
+  {
+	data.px[mia]= xs+ xt* s1x+ yt* s2x;
+	data.py[mia]= ys+ xt* s1y+ yt* s2y;
+	data.pz[mia]= zs+ xt* s1z+ yt* s2z;
+	data.pbi[mia]= xa;
+	data.t1x[mia]= s1x;
+	data.t1y[mia]= s1y;
+	data.t1z[mia]= s1z;
+	data.t2x[mia]= s2x;
+	data.t2y[mia]= s2y;
+	data.t2z[mia]= s2z;
+	data.psalp[mia]= saln;
+
+	if( ix == 2)
+	  yt= -yt;
+
+	if( (ix == 1) || (ix == 3) )
+	  xt= -xt;
+
+	mia++;
+  }
+
+  if( nx <= data.mp)
+	data.mp += 3;
+
+  if( ny > 0 )
+	data.pz[mi]=10000.0;
+
+  /* Process new patches created */
+  if( ! CHILD )
+	New_Patch_Data();
+
+  return;
+}
+
+/*-----------------------------------------------------------------------*/
+
 /* connect sets up segment connection data in arrays icon1 and */
 /* icon2 by searching for segment ends that are in contact. */
   gboolean
@@ -169,8 +281,8 @@ conect( int ignd )
   {
 	/* Allocate memory to connections */
 	mreq = (size_t)(data.n + data.m) * sizeof(int);
-	mem_realloc( (void *)&data.icon1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.icon2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.icon1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.icon2, mreq, "in geometry.c" );
 
 	for( i = 0; i < data.n; i++ )
 	{
@@ -194,7 +306,7 @@ conect( int ignd )
 		  fprintf( stderr,
 			  "xnec2c: conect(): geometry data error\n"
 			  "Segment %d extends below ground\n", iz );
-		  stop( _("conect(): Geometry data error\n"\
+		  stop( _("conect(): Geometry data error\n"
 				"Segment extends below ground"), ERR_OK );
 		  return( FALSE );
 		}
@@ -246,7 +358,7 @@ conect( int ignd )
 		  fprintf( stderr,
 			  "xnec2c: conect(): geometry data error\n"
 			  "segment %d extends below ground\n", iz );
-		  stop( _("conect(): Geometry data error\n"\
+		  stop( _("conect(): Geometry data error\n"
 				"Segment extends below ground"), ERR_OK );
 		  return( FALSE );
 		}
@@ -258,7 +370,7 @@ conect( int ignd )
 			fprintf( stderr,
 				"xnec2c: conect(): geometry data error\n"
 				"segment %d lies in ground plane\n", iz );
-			stop( _("conect(): Geometry data error\n"\
+			stop( _("conect(): Geometry data error\n"
 				  "Segment lies in ground plane"), ERR_OK );
 			return( FALSE );
 		  }
@@ -368,7 +480,7 @@ conect( int ignd )
 
   /* Allocate to connection buffers */
   mreq = (size_t)segj.maxcon * sizeof(int);
-  mem_realloc( (void *)&segj.jco, mreq, "in geometry.c" );
+  mem_realloc( (void **)&segj.jco, mreq, "in geometry.c" );
 
   /* Adjust connected segment ends to exactly coincide.
    * Also find old seg. connecting to new segment */
@@ -391,15 +503,6 @@ conect( int ignd )
 	  {
 		do
 		{
-		  if( ix == 0 )
-		  {
-			fprintf( stderr,
-				"xnec2c: connect(): segment connection"
-				" error for segment: %d\n", ix );
-			stop( _("connect(): Segment connection error"), ERR_OK );
-			return( FALSE );
-		  }
-
 		  if( ix < 0 )
 			ix= -ix;
 		  else
@@ -422,7 +525,7 @@ conect( int ignd )
 		  {
 			segj.maxcon = ic+1;
 			mreq = (size_t)segj.maxcon * sizeof(int);
-			mem_realloc( (void *)&segj.jco, mreq, "in geometry.c" );
+			mem_realloc( (void **)&segj.jco, mreq, "in geometry.c" );
 		  }
 		  segj.jco[ic-1]= ix* jend;
 
@@ -445,19 +548,18 @@ conect( int ignd )
 		while( ix != 0 );
 
 		if( jump && (iend == 1) ) break;
-		else
-		  if( jump )
-		  {
-			iend=1;
-			jend=1;
-			ix= data.icon2[j];
-			ic=1;
-			segj.jco[0]= jx;
-			xa= data.x2[j];
-			ya= data.y2[j];
-			za= data.z2[j];
-			continue;
-		  }
+		else if( jump )
+		{
+		  iend=1;
+		  jend=1;
+		  ix= data.icon2[j];
+		  ic=1;
+		  segj.jco[0]= jx;
+		  xa= data.x2[j];
+		  ya= data.y2[j];
+		  za= data.z2[j];
+		  continue;
+		}
 
 		sep= (double)ic;
 		xa= xa/ sep;
@@ -505,9 +607,9 @@ conect( int ignd )
   } /* for( j = 0; j < data.n; j++ ) */
 
   mreq = (size_t)segj.maxcon * sizeof(double);
-  mem_realloc( (void *)&segj.ax, mreq, "in geometry.c" );
-  mem_realloc( (void *)&segj.bx, mreq, "in geometry.c" );
-  mem_realloc( (void *)&segj.cx, mreq, "in geometry.c" );
+  mem_realloc( (void **)&segj.ax, mreq, "in geometry.c" );
+  mem_realloc( (void **)&segj.bx, mreq, "in geometry.c" );
+  mem_realloc( (void **)&segj.cx, mreq, "in geometry.c" );
 
   return( TRUE );
 }
@@ -539,17 +641,17 @@ helix(
 
   /* Reallocate tags buffer */
   mreq = (size_t)data.n * sizeof(int);
-  mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
   /* Reallocate wire buffers */
   mreq = (size_t)data.n * sizeof(double);
-  mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
   data.z1[ist] = 0.0;
   for( i = ist; i < data.n; i++ )
@@ -567,10 +669,10 @@ helix(
 	  if( b1 == 0.0)
 		b1= a1;
 
-	  data.x1[i]= a1* cos(2.0* PI* data.z1[i]/ s);
-	  data.y1[i]= b1* sin(2.0* PI* data.z1[i]/ s);
-	  data.x2[i]= a1* cos(2.0* PI* data.z2[i]/ s);
-	  data.y2[i]= b1* sin(2.0* PI* data.z2[i]/ s);
+	  data.x1[i]= a1* cos(2.0* M_PI* data.z1[i]/ s);
+	  data.y1[i]= b1* sin(2.0* M_PI* data.z1[i]/ s);
+	  data.x2[i]= a1* cos(2.0* M_PI* data.z2[i]/ s);
+	  data.y2[i]= b1* sin(2.0* M_PI* data.z2[i]/ s);
 	}
 	else
 	{
@@ -578,13 +680,13 @@ helix(
 		b2= a2;
 
 	  data.x1[i]=( a1+( a2- a1)* data.z1[i] /
-		  fabs( hl))* cos(2.0* PI* data.z1[i]/ s);
+		  fabs( hl))* cos(2.0* M_PI* data.z1[i]/ s);
 	  data.y1[i]=( b1+( b2- b1)* data.z1[i] /
-		  fabs( hl))* sin(2.0* PI* data.z1[i]/ s);
+		  fabs( hl))* sin(2.0* M_PI* data.z1[i]/ s);
 	  data.x2[i]=( a1+( a2- a1)* data.z2[i] /
-		  fabs( hl))* cos(2.0* PI* data.z2[i]/ s);
+		  fabs( hl))* cos(2.0* M_PI* data.z2[i]/ s);
 	  data.y2[i]=( b1+( b2- b1)* data.z2[i] /
-		  fabs( hl))* sin(2.0* PI* data.z2[i]/ s);
+		  fabs( hl))* sin(2.0* M_PI* data.z2[i]/ s);
 
 	} /* if( a2 == a1) */
 
@@ -617,7 +719,7 @@ isegno( int itagi, int mx)
 	fprintf( stderr,
 		"xnec2c: isegno(): check data, parameter specifying segment\n"
 		"position in a group of equal tags must not be zero\n" );
-	stop( _("isegno(): Parameter specifying segment\n"\
+	stop( _("isegno(): Parameter specifying segment\n"
 		  "position in a group of equal tags is zero"), ERR_OK );
 	return( -1 );
   }
@@ -710,17 +812,17 @@ move( double rox, double roy,
 	  k= data.n;
 	  /* Reallocate tags buffer */
 	  mreq = (size_t)(data.n + data.m + (data.n + 1 - i1) * nrpt) * sizeof(int);
-	  mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
 	  /* Reallocate wire buffers */
 	  mreq = (size_t)(data.n + (data.n + 1 - i1) * nrpt) * sizeof(double);
-	  mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 	}
 
 	for( ir = 0; ir < nrp; ir++ )
@@ -764,17 +866,17 @@ move( double rox, double roy,
 
 	/* Reallocate patch buffers */
 	mreq = (size_t)(data.m * (nrpt + 1)) * sizeof(double);
-	mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
 	for( ii = 0; ii < nrp; ii++ )
 	{
@@ -849,17 +951,17 @@ patch(
 
   /* Reallocate patch buffers */
   mreq = (size_t)data.m * sizeof(double);
-  mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2y, mreq, "in geometry.c");
-  mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2y, mreq, "in geometry.c");
+  mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
   if( nx > 0) ntp=2;
   else ntp= ny;
@@ -964,7 +1066,7 @@ patch(
 		  fprintf( stderr,
 			  "xnec2c: patch(): corners of quadrilateral\n"
 			  "patch do not lie in a plane\n" );
-		  stop( _("patch(): Corners of quadrilateral\n"\
+		  stop( _("patch(): Corners of quadrilateral\n"
 				"patch do not lie in a plane"), ERR_OK );
 		  return( FALSE );
 		}
@@ -988,17 +1090,17 @@ patch(
 	data.m += nx*ny-1;
 	/* Reallocate patch buffers */
 	mreq = (size_t)data.m * sizeof(double);
-	mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
 	xn2= data.px[mi]- s1x- s2x;
 	yn2= data.py[mi]- s1y- s2y;
@@ -1046,121 +1148,6 @@ patch(
 
 /*-----------------------------------------------------------------------*/
 
-/*** this function was an 'entry point' (part of) 'patch()' ***/
-  void
-subph( int nx, int ny )
-{
-  int mia, ix, iy, mi;
-  size_t mreq;
-  double xs, ys, zs, xa, xst, s1x, s1y;
-  double s1z, s2x, s2y, s2z, saln, xt, yt;
-
-  /* Reallocate patch buffers */
-  if( ny == 0 ) data.m += 3;
-  else data.m += 4;
-
-  mreq = (size_t)data.m * sizeof(double);
-  mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
-  if( !CHILD )
-  {
-	mreq = (size_t)(data.n + 2 * data.m) * sizeof(GdkSegment);
-	mem_realloc( (void *)&structure_segs, mreq, "in geometry.c" );
-  }
-
-  /* Shift patches to make room for new ones */
-  if( (ny == 0) && (nx != data.m) )
-  {
-	for( iy = data.m-1; iy > nx+2; iy-- )
-	{
-	  ix = iy-3;
-	  data.px[iy]= data.px[ix];
-	  data.py[iy]= data.py[ix];
-	  data.pz[iy]= data.pz[ix];
-	  data.pbi[iy]= data.pbi[ix];
-	  data.psalp[iy]= data.psalp[ix];
-	  data.t1x[iy]= data.t1x[ix];
-	  data.t1y[iy]= data.t1y[ix];
-	  data.t1z[iy]= data.t1z[ix];
-	  data.t2x[iy]= data.t2x[ix];
-	  data.t2y[iy]= data.t2y[ix];
-	  data.t2z[iy]= data.t2z[ix];
-	}
-
-  } /* if( (ny == 0) || (nx != m) ) */
-
-  /* divide patch for connection */
-  mi= nx-1;
-  xs= data.px[mi];
-  ys= data.py[mi];
-  zs= data.pz[mi];
-  xa= data.pbi[mi]/4.0;
-  xst= sqrt( xa)/2.0;
-  s1x= data.t1x[mi];
-  s1y= data.t1y[mi];
-  s1z= data.t1z[mi];
-  s2x= data.t2x[mi];
-  s2y= data.t2y[mi];
-  s2z= data.t2z[mi];
-  saln= data.psalp[mi];
-  xt= xst;
-  yt= xst;
-
-  if( ny == 0)
-	mia= mi;
-  else
-  {
-	data.mp++;
-	mia= data.m-1;
-  }
-
-  for( ix = 1; ix <= 4; ix++ )
-  {
-	data.px[mia]= xs+ xt* s1x+ yt* s2x;
-	data.py[mia]= ys+ xt* s1y+ yt* s2y;
-	data.pz[mia]= zs+ xt* s1z+ yt* s2z;
-	data.pbi[mia]= xa;
-	data.t1x[mia]= s1x;
-	data.t1y[mia]= s1y;
-	data.t1z[mia]= s1z;
-	data.t2x[mia]= s2x;
-	data.t2y[mia]= s2y;
-	data.t2z[mia]= s2z;
-	data.psalp[mia]= saln;
-
-	if( ix == 2)
-	  yt= -yt;
-
-	if( (ix == 1) || (ix == 3) )
-	  xt= -xt;
-
-	mia++;
-  }
-
-  if( nx <= data.mp)
-	data.mp += 3;
-
-  if( ny > 0 )
-	data.pz[mi]=10000.0;
-
-  /* Process new patches created */
-  if( ! CHILD )
-	New_Patch_Data();
-
-  return;
-}
-
-/*-----------------------------------------------------------------------*/
-
 /* reflc reflects partial structure along x,y, or z axes */
 /* or rotates structure to complete a symmetric structure. */
   gboolean
@@ -1189,17 +1176,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 	  {
 		/* Reallocate tags buffer */
 		mreq = (size_t)(2 * data.n + data.m) * sizeof(int);
-		mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
 		/* Reallocate wire buffers */
 		mreq = (size_t)(2 * data.n) * sizeof(double);
-		mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
 		for( i = 0; i < data.n; i++ )
 		{
@@ -1212,7 +1199,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 			fprintf( stderr,
 				"xnec2c: reflc(): geometry data error\n"
 				"segment %d lies in plane of symmetry\n", i+1 );
-			stop( _("reflc(): Geometry data error\n"\
+			stop( _("reflc(): Geometry data error\n"
 				  "Segment lies in plane of symmetry"), ERR_OK );
 			return( FALSE );
 		  }
@@ -1243,17 +1230,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 	  {
 		/* Reallocate patch buffers */
 		mreq = (size_t)(2 * data.m) * sizeof(double);
-		mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
 		for( i = 0; i < data.m; i++ )
 		{
@@ -1263,7 +1250,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 			fprintf( stderr,
 				"xnec2c: reflc(): geometry data error\n"
 				"patch %d lies in plane of symmetry\n", i+1 );
-			stop( _("reflc(): Geometry data error\n"\
+			stop( _("reflc(): Geometry data error\n"
 				  "Patch lies in plane of symmetry"), ERR_OK );
 			return( FALSE );
 		  }
@@ -1294,16 +1281,16 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 	  {
 		/* Reallocate tags buffer */
 		mreq = (size_t)(2 * data.n) * sizeof(int);
-		mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 		/* Reallocate wire buffers */
 		mreq = (size_t)(2 * data.n) * sizeof(double);
-		mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
 		for( i = 0; i < data.n; i++ )
 		{
@@ -1316,7 +1303,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 			fprintf( stderr,
 				"xnec2c: reflc(): geometry data error\n"
 				"segment %d lies in plane of symmetry\n", i+1 );
-			stop( _("reflc(): Geometry data error\n"\
+			stop( _("reflc(): Geometry data error\n"
 				  "Segment lies in plane of symmetry"), ERR_OK );
 			return( FALSE );
 		  }
@@ -1347,17 +1334,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 	  {
 		/* Reallocate patch buffers */
 		mreq = (size_t)(2 * data.m) * sizeof(double);
-		mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-		mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+		mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
 		for( i = 0; i < data.m; i++ )
 		{
@@ -1367,7 +1354,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 			fprintf( stderr,
 				"xnec2c: reflc(): geometry data error\n"
 				"patch %d lies in plane of symmetry\n", i+1 );
-			stop( _("reflc(): Geometry data error\n"\
+			stop( _("reflc(): Geometry data error\n"
 				  "Patch lies in plane of symmetry"), ERR_OK );
 			return( FALSE );
 		  }
@@ -1399,17 +1386,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 	{
 	  /* Reallocate tags buffer */
 	  mreq = (size_t)(2 * data.n) * sizeof(int);
-	  mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
 	  /* Reallocate wire buffers */
 	  mreq = (size_t)(2 * data.n) * sizeof(double);
-	  mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-	  mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+	  mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
 	  for( i = 0; i < data.n; i++ )
 	  {
@@ -1422,7 +1409,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 		  fprintf( stderr,
 			  "xnec2c: reflc(): geometry data error\n"
 			  "segment %d lies in plane of symmetry\n", i+1 );
-		  stop( _("reflc(): Geometry data error\n"\
+		  stop( _("reflc(): Geometry data error\n"
 				"Segment lies in plane of symmetry"), ERR_OK );
 		  return( FALSE );
 		}
@@ -1451,17 +1438,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 
 	/* Reallocate patch buffers */
 	mreq = (size_t)(2 * data.m) * sizeof(double);
-	mem_realloc( (void *)&data.px,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.px,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
 	for( i = 0; i < data.m; i++ )
 	{
@@ -1471,7 +1458,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 		fprintf( stderr,
 			"xnec2c: reflc(): geometry data error\n"
 			"patch %d lies in plane of symmetry\n", i+1 );
-		stop( _("reflc(): Geometry data error\n"\
+		stop( _("reflc(): Geometry data error\n"
 			  "Patch lies in plane of symmetry"), ERR_OK );
 		return( FALSE );
 	  }
@@ -1497,7 +1484,7 @@ reflc( int ix, int iy, int iz, int iti, int nop )
   /* reproduce structure with rotation to form cylindrical structure */
   fnop= (double)nop;
   data.ipsym=-1;
-  sam=TP/ fnop;
+  sam=M_2PI/ fnop;
   cs= cos( sam);
   ss= sin( sam);
 
@@ -1508,17 +1495,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 
 	/* Reallocate tags buffer */
 	mreq = (size_t)data.n * sizeof(int);
-	mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
 	/* Reallocate wire buffers */
 	mreq = (size_t)data.n * sizeof(double);
-	mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-	mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+	mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
 	for( i = nx; i < data.n; i++ )
 	{
@@ -1551,17 +1538,17 @@ reflc( int ix, int iy, int iz, int iti, int nop )
 
   /* Reallocate patch buffers */
   mreq = (size_t)data.m * sizeof(double);
-  mem_realloc( (void *)&data.px,  mreq, "in geometry.c"  );
-  mem_realloc( (void *)&data.py,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.pz,  mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1x, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1y, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t1z, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2x, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2y, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.t2z, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.pbi, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.psalp, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.px,  mreq, "in geometry.c"  );
+  mem_realloc( (void **)&data.py,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.pz,  mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1x, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1y, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t1z, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2x, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2y, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.t2z, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.pbi, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.psalp, mreq, "in geometry.c" );
 
   for( i = nx; i < data.m; i++ )
   {
@@ -1613,17 +1600,17 @@ wire( double xw1, double yw1, double zw1,
 
   /* Reallocate tags buffer */
   mreq = (size_t)data.n * sizeof(int);
-  mem_realloc( (void *)&data.itag, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.itag, mreq, "in geometry.c" );
 
   /* Reallocate wire buffers */
   mreq = (size_t)data.n * sizeof(double);
-  mem_realloc( (void *)&data.x1, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.y1, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.z1, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.x2, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.y2, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.z2, mreq, "in geometry.c" );
-  mem_realloc( (void *)&data.bi, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.x1, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.y1, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.z1, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.x2, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.y2, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.z2, mreq, "in geometry.c" );
+  mem_realloc( (void **)&data.bi, mreq, "in geometry.c" );
 
   xd= xw2- xw1;
   yd= yw2- yw1;
