@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,7 +18,6 @@
 #define	INPUT_H		1
 
 #include "common.h"
-#include "support.h"
 
 /* Command Mnemonics */
 enum CMDN_MNM
@@ -44,15 +40,14 @@ enum CMDN_MNM
   RP,
   SY,
   TL,
-  XQ
+  XQ,
+  NUM_CMNDS
 };
 
-#define NUM_CMNDS	19
-
 /* Geometry Mnemonics */
 enum GEOM_MNM
 {
-  GW = 1,
+  GW = 0,
   GX,
   GR,
   GS,
@@ -63,10 +58,10 @@ enum GEOM_MNM
   GA,
   SC,
   GH,
-  GF
+  GF,
+  CT,
+  NUM_GEOMN
 };
 
-#define NUM_GEOMN	13
-
 #endif
 
