@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -23,9 +20,9 @@
 #include "common.h"
 #include "callbacks.h"
 #include "interface.h"
-#include "support.h"
 #include "fork.h"
 #include "nec2_model.h"
+#include <sys/wait.h>
 
 #define OPEN_INPUT_FLAGS \
   RDPAT_FLAGS		| \
