@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -17,66 +14,48 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* nec2_model.c
- *
- * Structure modelling functions for xnec2c
- */
-
 #include "nec2_model.h"
 #include "shared.h"
 
 /*------------------------------------------------------------------------*/
 
-/* Nec2_Input_File_Treeview()
+/* Insert_Columns()
  *
- * Reads a NEC2 input file and renders it in a tree view
+ * Inserts columns in a list store
  */
-  void
-Nec2_Input_File_Treeview( int action )
+  static void
+Insert_Columns(
+	GtkBuilder *builder,
+	gchar *treeview,
+	GtkListStore* store,
+	int ncols,
+	char *colname[] )
 {
-  /* Abort if editor window is not opened */
-  if( nec2_edit_window == NULL ) return;
+  int idx;
+  GtkTreeModel *model;
+  GtkCellRenderer *renderer;
 
-  /* Signal save of edited file */
-  SetFlag( NEC2_EDIT_SAVE );
-
-  /* Implement user action */
-  switch( action )
+  static GtkWidget *view;
+  view = Builder_Get_Object( builder, treeview );
+  for( idx = 0; idx < ncols; idx++ )
   {
-	case NEC2_EDITOR_REVERT: /* Revert editor to file contents */
-	  /* Clear all tree view list stores */
-	  gtk_list_store_clear( cmnt_store );
-	  gtk_list_store_clear( geom_store );
-	  gtk_list_store_clear( cmnd_store );
-	  break;
+	renderer = gtk_cell_renderer_text_new();
+	g_object_set(renderer, "editable", TRUE, NULL);
+	g_signal_connect( renderer, "edited",
+		(GCallback)cell_edited_callback, view );
+	g_object_set_data( G_OBJECT(renderer),
+		"column", GUINT_TO_POINTER(idx) );
+	gtk_tree_view_insert_column_with_attributes(
+		GTK_TREE_VIEW(view), -1, colname[idx],
+		renderer, "text", idx, NULL );
+  }
+  model = GTK_TREE_MODEL(store);
+  gtk_tree_view_set_model( GTK_TREE_VIEW (view), model );
 
-	case NEC2_EDITOR_NEW: /* Create new default input file */
-	  /* If tree view stores are already
-	   * created, just make the new file */
-	  Create_List_Stores(); /* Only done if needed */
-	  Create_Default_File();
-	  return;
+  /* Destroy model automatically with view */
+  g_object_unref( model );
 
-	case NEC2_EDITOR_RELOAD: /* Just reload input file */
-	  Create_List_Stores();  /* Only done if needed */
-	  break;
-
-  } /* switch( action ) */
-
-  /* Rewind NEC2 input file */
-  rewind( input_fp );
-
-  /*** List Comment cards ***/
-  List_Comments();
-
-  /*** List Geometry cards ***/
-  List_Geometry();
-
-  /*** Read Command cards ***/
-  List_Commands();
-
-  return;
-} /* Nec2_Input_File_Treeview() */
+} /* Insert_Columns() */
 
 /*------------------------------------------------------------------------*/
 
@@ -84,7 +63,7 @@ Nec2_Input_File_Treeview( int action )
  *
  * Create stores needed for the treeview
  */
-  void
+  static void
 Create_List_Stores( void )
 {
   /* Comments column names */
@@ -127,31 +106,28 @@ Create_List_Stores( void )
 
   /* Insert comment columns */
   Insert_Columns(
-	  nec2_edit_window, "nec2_cmnt_treeview",
+	  nec2_editor_builder, "nec2_cmnt_treeview",
 	  cmnt_store, CMNT_NUM_COLS, cmnt_col_name );
 
   /* Insert geometry columns */
   Insert_Columns(
-	  nec2_edit_window, "nec2_geom_treeview",
+	  nec2_editor_builder, "nec2_geom_treeview",
 	  geom_store, GEOM_NUM_COLS, geom_col_name );
 
   /* Insert command columns */
   Insert_Columns(
-	  nec2_edit_window, "nec2_cmnd_treeview",
+	  nec2_editor_builder, "nec2_cmnd_treeview",
 	  cmnd_store, CMND_NUM_COLS, cmnd_col_name );
 
   /* Set models to treviews */
-  gtk_tree_view_set_model(
-	  GTK_TREE_VIEW(lookup_widget(
-		  nec2_edit_window, "nec2_cmnt_treeview")),
+  gtk_tree_view_set_model( GTK_TREE_VIEW(
+		Builder_Get_Object(nec2_editor_builder, "nec2_cmnt_treeview")),
 	  GTK_TREE_MODEL(cmnt_store) );
-  gtk_tree_view_set_model(
-	  GTK_TREE_VIEW(lookup_widget(
-		  nec2_edit_window, "nec2_geom_treeview")),
+  gtk_tree_view_set_model( GTK_TREE_VIEW(
+		Builder_Get_Object(nec2_editor_builder, "nec2_geom_treeview")),
 	  GTK_TREE_MODEL(geom_store) );
-  gtk_tree_view_set_model(
-	  GTK_TREE_VIEW(lookup_widget(
-		  nec2_edit_window, "nec2_cmnd_treeview")),
+  gtk_tree_view_set_model( GTK_TREE_VIEW(
+		Builder_Get_Object(nec2_editor_builder, "nec2_cmnd_treeview")),
 	  GTK_TREE_MODEL(cmnd_store) );
 
 } /* Create_List_Stores() */
@@ -162,12 +138,13 @@ Create_List_Stores( void )
  *
  * Creates a default NEC2 file if needed
  */
-  void
+  static void
 Create_Default_File( void )
 {
   GtkTreeIter iter;
-  int idx;
-  char str[55];
+  int idx, idi;
+  char str[64];
+  size_t s = sizeof( str );
 
 
   /* Clear all tree views */
@@ -176,9 +153,9 @@ Create_Default_File( void )
   gtk_list_store_clear( cmnd_store );
 
   /* Append a default comment row */
-  strcpy( str, _("--- NEC2 Input File created by ") );
-  strcat( str, PACKAGE_STRING);
-  strcat( str, " ---" );
+  Strlcpy( str, _("--- NEC2 Input File created or edited by "), s );
+  Strlcat( str, PACKAGE_STRING, s );
+  Strlcat( str, " ---", s );
   gtk_list_store_append( cmnt_store, &iter );
   gtk_list_store_set(
 	  cmnt_store, &iter,
@@ -287,7 +264,7 @@ Create_Default_File( void )
 	  CMND_COL_I1, "0",
 	  CMND_COL_I2, "19",
 	  CMND_COL_I3, "37",
-	  CMND_COL_I4, "0",
+	  CMND_COL_I4, "1000",
 	  CMND_COL_F1, "0.0",
 	  CMND_COL_F2, "0.0",
 	  CMND_COL_F3, "10.0",
@@ -300,8 +277,8 @@ Create_Default_File( void )
   gtk_list_store_append( cmnd_store, &iter );
   gtk_list_store_set( cmnd_store,
 	  &iter, CMND_COL_NAME, "EN", -1 );
-  for( idx = CMND_COL_I1; idx < CMND_NUM_COLS; idx++ )
-	gtk_list_store_set( cmnd_store, &iter, idx, "0", -1 );
+  for( idi = CMND_COL_I1; idi < CMND_NUM_COLS; idi++ )
+	gtk_list_store_set( cmnd_store, &iter, idi, "0", -1 );
 
 } /* Create_Default_File() */
 
@@ -311,7 +288,7 @@ Create_Default_File( void )
  *
  * Reads comments from file and lists in tree view
  */
-  void
+  static void
 List_Comments( void )
 {
   GtkTreeIter iter;
@@ -328,18 +305,18 @@ List_Comments( void )
   do
   {
 	/* Read a line from input file */
-	if( load_line(line_buf, input_fp) == EOF )
-	  stop( _("List_Comments():\n"\
-			"Error reading input file\n"\
+	if( Load_Line(line_buf, input_fp) == EOF )
+	  stop( _("List_Comments():\n"
+			"Error reading input file\n"
 			"Unexpected EOF (End of File)"), ERR_OK );
 
 	/* Check for short or missing CM or CE and fix */
 	if( strlen(line_buf) < 2 )
 	{
-	  stop( _("List_Comments():\n"\
-			"Error reading input file\n"\
+	  stop( _("List_Comments():\n"
+			"Error reading input file\n"
 			"Comment mnemonic short or missing"), ERR_OK );
-	  strcpy( line_buf, "XX " );
+	  Strlcpy( line_buf, "XX ", sizeof(line_buf) );
 	}
 
 	/* If only mnemonic in card,
@@ -347,8 +324,7 @@ List_Comments( void )
 	if( strlen(line_buf) == 2 ) line_buf[3] = '\0';
 
 	/* Separate card's id mnemonic */
-	strncpy( ain, line_buf, 2 );
-	ain[2] = '\0';
+	Strlcpy( ain, line_buf, sizeof(ain) );
 
 	/* Append a comment row and fill in text if opening call */
 	if( !ret )
@@ -374,7 +350,7 @@ List_Comments( void )
  *
  * Reads geometry cards from file and lists in tree view
  */
-  void
+  static void
 List_Geometry( void )
 {
   GtkTreeIter iter;
@@ -400,16 +376,17 @@ List_Geometry( void )
   do
   {
 	/* Read a geometry card. Errors are handled in readgm() */
-	readgm( ain, &iv[0], &iv[1],
+	if( !readgm( ain, &iv[0], &iv[1],
 		&fv[0], &fv[1], &fv[2], &fv[3],
-		&fv[4], &fv[5], &fv[6] );
+		&fv[4], &fv[5], &fv[6]) )
+	  break;
 
 	/* Ignore in-data (NEC4 style) comments */
 	if( strcmp(ain, "CM") == 0 ) continue;
 
 	/* Format card data and print to string */
 	snprintf( si[0], 6, "%5d",  iv[0] );
-	snprintf( si[1], 6, "%5d ", iv[1] );
+	snprintf( si[1], 7, "%5d ", iv[1] );
 	for( idx = GEOM_COL_F1; idx <= GEOM_COL_F7; idx++ )
 	  snprintf( sf[idx-GEOM_COL_F1], 13,
 		  "%12.5E", (double)fv[idx-GEOM_COL_F1] );
@@ -442,7 +419,7 @@ List_Geometry( void )
  *
  * Reads command cards from file and lists in tree view
  */
-  void
+  static void
 List_Commands( void )
 {
   GtkTreeIter iter;
@@ -509,36 +486,56 @@ List_Commands( void )
 
 /*------------------------------------------------------------------------*/
 
-/* Inserts columns in a list store */
+/* Nec2_Input_File_Treeview()
+ *
+ * Reads a NEC2 input file and renders it in a tree view
+ */
   void
-Insert_Columns(	GtkWidget *window, gchar *treeview,
-	GtkListStore* store, int ncols, char *colname[] )
+Nec2_Input_File_Treeview( int action )
 {
-  int idx;
-  GtkTreeModel *model;
-  GtkCellRenderer *renderer;
+  /* Abort if editor window is not opened */
+  if( nec2_edit_window == NULL ) return;
 
-  static GtkWidget *view;
-  view = lookup_widget( window, treeview );
-  for( idx = 0; idx < ncols; idx++ )
+  /* Signal save of edited file */
+  SetFlag( NEC2_EDIT_SAVE );
+
+  /* Implement user action */
+  switch( action )
   {
-	renderer = gtk_cell_renderer_text_new();
-	g_object_set(renderer, "editable", TRUE, NULL);
-	g_signal_connect( renderer, "edited",
-		(GCallback)cell_edited_callback, view );
-	g_object_set_data( G_OBJECT(renderer),
-		"column", GUINT_TO_POINTER(idx) );
-	gtk_tree_view_insert_column_with_attributes(
-		GTK_TREE_VIEW(view), -1, colname[idx],
-		renderer, "text", idx, NULL );
-  }
-  model = GTK_TREE_MODEL(store);
-  gtk_tree_view_set_model( GTK_TREE_VIEW (view), model );
+	case NEC2_EDITOR_REVERT: /* Revert editor to file contents */
+	  /* Clear all tree view list stores */
+	  gtk_list_store_clear( cmnt_store );
+	  gtk_list_store_clear( geom_store );
+	  gtk_list_store_clear( cmnd_store );
+	  break;
 
-  /* Destroy model automatically with view */
-  g_object_unref( model );
+	case NEC2_EDITOR_NEW: /* Create new default input file */
+	  /* If tree view stores are already
+	   * created, just make the new file */
+	  Create_List_Stores(); /* Only done if needed */
+	  Create_Default_File();
+	  return;
 
-} /* Insert_Columns() */
+	case NEC2_EDITOR_RELOAD: /* Just reload input file */
+	  Create_List_Stores();  /* Only done if needed */
+	  break;
+
+  } /* switch( action ) */
+
+  /* Rewind NEC2 input file */
+  rewind( input_fp );
+
+  /*** List Comment cards ***/
+  List_Comments();
+
+  /*** List Geometry cards ***/
+  List_Geometry();
+
+  /*** Read Command cards ***/
+  List_Commands();
+
+  return;
+} /* Nec2_Input_File_Treeview() */
 
 /*------------------------------------------------------------------------*/
 
@@ -581,12 +578,59 @@ cell_edited_callback(
   }
   else
 	gtk_list_store_set( GTK_LIST_STORE(model),
-		&iter, column, new_text, -1 );
+	  &iter, column, new_text, -1 );
 
 } /* cell_edited_callback() */
 
 /*------------------------------------------------------------------------*/
 
+/* Save_Treeview_Data()
+ *
+ * Saves tree view data to an open NEC2 input file
+ */
+  static void
+Save_Treeview_Data( GtkTreeView *tree_view, int ncols, FILE *nec2_fp )
+{
+  GtkTreeModel *list_store;
+  GtkTreeIter iter;
+  gboolean valid;
+  int idx;
+
+  /* Abort if no open file to sane to */
+  if( nec2_fp == NULL )
+  {
+	stop( _("Cannot save treeview data\n"
+		  "Please use the Save button\n"
+		  "to specify a file path"), ERR_STOP );
+  }
+
+  /* Get the first iter in the list */
+  list_store = GTK_TREE_MODEL( gtk_tree_view_get_model(tree_view) );
+  valid = gtk_tree_model_get_iter_first( list_store, &iter );
+
+  /* Walk through all rows and print data to file */
+  while( valid )
+  {
+	gchar *str_data;
+
+	for( idx = 0; idx < ncols; idx++ )
+	{
+	  gtk_tree_model_get( list_store, &iter, idx, &str_data, -1 );
+	  fprintf( nec2_fp, "%s ", str_data );
+	  g_free( str_data );
+	}
+
+	/* Overwrite last space with newline */
+	if( fseek(nec2_fp, -1, SEEK_CUR) == 0 )
+	  fprintf( nec2_fp, "\n" );
+
+	valid = gtk_tree_model_iter_next( list_store, &iter );
+  } /* while( valid ) */
+
+} /* Save_Treeview_Data() */
+
+/*------------------------------------------------------------------------*/
+
 /* Save_Nec2_Input_File()
  *
  * Saves the data in a NEC2 input treeview to a given filename
@@ -605,18 +649,18 @@ Save_Nec2_Input_File( GtkWidget *treeview_window, char *nec2_file )
   if( !Open_File(&nec2_fp, nec2_file, "w") ) return;
 
   /* Save comments to file */
-  tree_view = GTK_TREE_VIEW( lookup_widget(
-		treeview_window, "nec2_cmnt_treeview") );
+  tree_view = GTK_TREE_VIEW( Builder_Get_Object(
+		nec2_editor_builder, "nec2_cmnt_treeview") );
   Save_Treeview_Data( tree_view, CMNT_NUM_COLS, nec2_fp );
 
   /* Save geometry to file */
-  tree_view = GTK_TREE_VIEW( lookup_widget(
-		treeview_window, "nec2_geom_treeview") );
+  tree_view = GTK_TREE_VIEW( Builder_Get_Object(
+		nec2_editor_builder, "nec2_geom_treeview") );
   Save_Treeview_Data( tree_view, GEOM_NUM_COLS, nec2_fp );
 
   /* Save commands to file */
-  tree_view = GTK_TREE_VIEW( lookup_widget(
-		treeview_window, "nec2_cmnd_treeview") );
+  tree_view = GTK_TREE_VIEW( Builder_Get_Object(
+		nec2_editor_builder, "nec2_cmnd_treeview") );
   Save_Treeview_Data( tree_view, CMND_NUM_COLS, nec2_fp );
 
   /* Re-open file in read mode */
@@ -626,67 +670,3 @@ Save_Nec2_Input_File( GtkWidget *treeview_window, char *nec2_file )
 
 /*------------------------------------------------------------------------*/
 
-/* Save_Treeview_Data()
- *
- * Saves tree view data to an open NEC2 input file
- */
-  void
-Save_Treeview_Data( GtkTreeView *tree_view, int ncols, FILE *nec2_fp )
-{
-  GtkTreeModel *list_store;
-  GtkTreeIter iter;
-  gboolean valid;
-  int idx;
-
-  /* Abort if no open file to sane to */
-  if( nec2_fp == NULL )
-  {
-	stop( _("Cannot save treeview data\n"\
-		  "Please use the Save button\n"\
-		  "to specify a file path"), ERR_STOP );
-  }
-
-  /* Get the first iter in the list */
-  list_store = GTK_TREE_MODEL( gtk_tree_view_get_model(tree_view) );
-  valid = gtk_tree_model_get_iter_first( list_store, &iter );
-
-  /* Walk through all rows and print data to file */
-  while( valid )
-  {
-	gchar *str_data;
-
-	for( idx = 0; idx < ncols; idx++ )
-	{
-	  gtk_tree_model_get( list_store, &iter, idx, &str_data, -1 );
-	  fprintf( nec2_fp, "%s ", str_data );
-	  g_free( str_data );
-	}
-
-	/* Overwrite last space with newline */
-	fseek( nec2_fp, -1, SEEK_CUR );
-	fprintf( nec2_fp, "\n" );
-
-	valid = gtk_tree_model_iter_next( list_store, &iter );
-  } /* while( valid ) */
-
-} /* Save_Treeview_Data() */
-
-/*------------------------------------------------------------------------*/
-
-/* Helper function */
-  gboolean
-gtk_tree_model_iter_previous(GtkTreeModel *tree_model, GtkTreeIter *iter)
-{
-  GtkTreePath *path;
-  gboolean ret;
-
-  path = gtk_tree_model_get_path (tree_model, iter);
-  ret = gtk_tree_path_prev (path);
-  if (ret == TRUE)
-	gtk_tree_model_get_iter (tree_model, iter, path);
-  gtk_tree_path_free (path);
-  return ret;
-}
-
-/*------------------------------------------------------------------------*/
-
