@@ -1,7 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *  Copyright (C) 2003-2010 N. Kyriazis neoklis.kyriazis(at)gmail.com
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -46,11 +43,56 @@
 #include "shared.h"
 
 /* Radiation pattern data */
+
+/*-----------------------------------------------------------------------*/
+
+/* calculates the xyz components of the electric */
+/* field due to surface currents */
+static void
+fflds( double rox, double roy, double roz,
+	complex double *scur, complex double *ex,
+	complex double *ey, complex double *ez )
+{
+  double *xs, *ys, *zs, *s;
+  int j, i, k;
+  double arg;
+  complex double ct;
+
+  xs = data.px;
+  ys = data.py;
+  zs = data.pz;
+  s = data.pbi;
+
+  *ex=CPLX_00;
+  *ey=CPLX_00;
+  *ez=CPLX_00;
+
+  i= -1;
+  for( j = 0; j < data.m; j++ )
+  {
+	i++;
+	arg= M_2PI*( rox* xs[i]+ roy* ys[i]+ roz* zs[i]);
+	ct= cmplx( cos( arg)* s[i], sin( arg)* s[i]);
+	k=3*j;
+	*ex += scur[k  ]* ct;
+	*ey += scur[k+1]* ct;
+	*ez += scur[k+2]* ct;
+  }
+
+  ct= rox* *ex+ roy* *ey+ roz* *ez;
+  *ex= CONST4*( ct* rox- *ex);
+  *ey= CONST4*( ct* roy- *ey);
+  *ez= CONST4*( ct* roz- *ez);
+
+  return;
+}
+
 /*-----------------------------------------------------------------------*/
 
 /* ffld calculates the far zone radiated electric fields, */
 /* the factor exp(j*k*r)/(r/lamda) not included */
-void ffld( double thet, double phi,
+static void
+ffld( double thet, double phi,
 	complex double *eth, complex double *eph )
 {
   int k, i, ip, jump;
@@ -95,7 +137,6 @@ void ffld( double thet, double phi,
 		  zrsin= csqrt(1.0- gnd.zrati* gnd.zrati* thz* thz);
 		  rrv=-( roz- gnd.zrati* zrsin)/( roz+ gnd.zrati* zrsin);
 		  rrh=( gnd.zrati* roz- zrsin)/( gnd.zrati* roz+ zrsin);
-
 		} /* if( gnd.iperf == 1) */
 
 		/* for the cliff problem, two reflection coefficients calculated */
@@ -110,7 +151,7 @@ void ffld( double thet, double phi,
 			zrsin= csqrt(1.0- gnd.zrati2* gnd.zrati2* thz* thz);
 			rrv2=-( roz- gnd.zrati2* zrsin)/( roz+ gnd.zrati2* zrsin);
 			rrh2=( gnd.zrati2* roz- zrsin)/( gnd.zrati2* roz+ zrsin);
-			darg= -TP*2.0* gnd.ch* roz;
+			darg= -M_2PI*2.0* gnd.ch* roz;
 		  }
 		} /* if( gnd.ifar > 1) */
 
@@ -128,9 +169,8 @@ void ffld( double thet, double phi,
 	  /* loop over structure segments */
 	  for( i = 0; i < data.n; i++ )
 	  {
-		omega=-( rox* data.cab[i] +
-			roy* data.sab[i]+ roz* data.salp[i]);
-		el= PI* data.si[i];
+		omega=-( rox* data.cab[i] +	roy* data.sab[i]+ roz* data.salp[i]);
+		el= M_PI* data.si[i];
 		sill= omega* el;
 		top= el+ sill;
 		bot= el- sill;
@@ -154,7 +194,7 @@ void ffld( double thet, double phi,
 		c= el*( boo+ too);
 		rr= a* crnt.air[i]+ b* crnt.bii[i]+ c* crnt.cir[i];
 		ri= a* crnt.aii[i]- b* crnt.bir[i]+ c* crnt.cii[i];
-		arg= TP*( data.x[i]* rox+ data.y[i]* roy+ data.z[i]* roz);
+		arg= M_2PI*( data.x[i]* rox+ data.y[i]* roy+ data.z[i]* roz);
 
 		if( (k != 1) || (gnd.ifar < 2) )
 		{
@@ -171,7 +211,6 @@ void ffld( double thet, double phi,
 
 		/* specular point distance */
 		dr= data.z[i]* tthet;
-
 		d= dr* phy+ data.x[i];
 		if( gnd.ifar == 2)
 		{
@@ -286,8 +325,8 @@ void ffld( double thet, double phi,
 	  jump = TRUE;
 	else
 	{
-	  *eth=( cix* thx+ ciy* thy+ ciz* thz)* CONST3;
-	  *eph=( cix* phx+ ciy* phy)* CONST3;
+	  *eth = ( cix * thx + ciy * thy + ciz * thz ) * CONST3;
+	  *eph = ( cix * phx + ciy * phy ) * CONST3;
 	  return;
 	}
 
@@ -343,53 +382,12 @@ void ffld( double thet, double phi,
 
   } /* for( ip = 0; ip < gnd.ksymp; ip++ ) */
 
-  ex= ex+ cix* CONST3;
-  ey= ey+ ciy* CONST3;
-  ez= ez+ ciz* CONST3;
-  *eth= ex* thx+ ey* thy+ ez* thz;
-  *eph= ex* phx+ ey* phy;
+  ex = ex + cix * CONST3;
+  ey = ey + ciy * CONST3;
+  ez = ez + ciz * CONST3;
 
-  return;
-}
-
-/*-----------------------------------------------------------------------*/
-
-/* calculates the xyz components of the electric */
-/* field due to surface currents */
-void fflds( double rox, double roy, double roz,
-	complex double *scur, complex double *ex,
-	complex double *ey, complex double *ez )
-{
-  double *xs, *ys, *zs, *s;
-  int j, i, k;
-  double arg;
-  complex double ct;
-
-  xs = data.px;
-  ys = data.py;
-  zs = data.pz;
-  s = data.pbi;
-
-  *ex=CPLX_00;
-  *ey=CPLX_00;
-  *ez=CPLX_00;
-
-  i= -1;
-  for( j = 0; j < data.m; j++ )
-  {
-	i++;
-	arg= TP*( rox* xs[i]+ roy* ys[i]+ roz* zs[i]);
-	ct= cmplx( cos( arg)* s[i], sin( arg)* s[i]);
-	k=3*j;
-	*ex += scur[k  ]* ct;
-	*ey += scur[k+1]* ct;
-	*ez += scur[k+2]* ct;
-  }
-
-  ct= rox* *ex+ roy* *ey+ roz* *ez;
-  *ex= CONST4*( ct* rox- *ex);
-  *ey= CONST4*( ct* roy- *ey);
-  *ez= CONST4*( ct* roz- *ez);
+  *eth = ex * thx + ey * thy + ez * thz;
+  *eph = ex * phx + ey * phy;
 
   return;
 }
@@ -397,7 +395,8 @@ void fflds( double rox, double roy, double roz,
 /*-----------------------------------------------------------------------*/
 
 /* gfld computes the radiated field including ground wave. */
-void gfld( double rho, double phi, double rz,
+static void
+gfld( double rho, double phi, double rz,
 	complex double *eth, complex double *epi,
 	complex double *erd, complex double ux, int ksymp )
 {
@@ -418,10 +417,10 @@ void gfld( double rho, double phi, double rz,
 	if( rz >= 1.0e-20)
 	  thet= atan( rho/ rz);
 	else
-	  thet= PI*.5;
+	  thet= M_PI*.5;
 
 	ffld( thet, phi, eth, epi);
-	arg= -TP* r;
+	arg= -M_2PI* r;
 	exa= cmplx( cos( arg), sin( arg))/ r;
 	*eth= *eth* exa;
 	*epi= *epi* exa;
@@ -477,7 +476,7 @@ void gfld( double rho, double phi, double rz,
 	  sph= rhy;
 	}
 
-	el= PI* data.si[i];
+	el= M_PI* data.si[i];
 	rfl=-1.0;
 
 	/* integration of (current)*(phase factor)
@@ -517,9 +516,9 @@ void gfld( double rho, double phi, double rz,
 		b* crnt.bii[i]+ c* crnt.cir[i];
 	  ri= a* crnt.aii[i] -
 		b* crnt.bir[i]+ c* crnt.cii[i];
-	  arg= TP*( data.x[i] *
+	  arg= M_2PI*( data.x[i] *
 		  rnx+ data.y[i]* rny+ data.z[i]* rnz* rfl);
-	  exa= cmplx( cos( arg), sin( arg))* cmplx( rr, ri)/ TP;
+	  exa= cmplx( cos( arg), sin( arg))* cmplx( rr, ri)/ M_2PI;
 
 	  if( k != 1 )
 	  {
@@ -549,7 +548,7 @@ void gfld( double rho, double phi, double rz,
 
   } /* for( i = 0; i < n; i++ ) */
 
-  arg= -TP* r;
+  arg= -M_2PI* r;
   exa= cmplx( cos( arg), sin( arg));
   cix= cix* exa;
   ciy= ciy* exa;
@@ -570,7 +569,8 @@ void gfld( double rho, double phi, double rz,
 /*-----------------------------------------------------------------------*/
 
 /* compute radiation pattern, gain, normalized gain */
-void rdpat(void)
+void
+rdpat( void )
 {
   int kth, kph, isens;
   double  prad, gcon, gcop;
@@ -588,37 +588,31 @@ void rdpat(void)
   {
 	gnd.cl= fpat.clt/ data.wlam;
 	gnd.ch= fpat.cht/ data.wlam;
-	gnd.zrati2= csqrt(1.0/ cmplx(
-		  fpat.epsr2,- fpat.sig2* data.wlam*59.96));
+	gnd.zrati2= csqrt(1.0/ cmplx(fpat.epsr2,- fpat.sig2* data.wlam*59.96));
   }
 
   /* Calculate radiation pattern data */
   /*** For applied voltage excitation ***/
   if( (fpat.ixtyp == 0) || (fpat.ixtyp == 5) )
   {
-	gcop= data.wlam* data.wlam* 2.0* PI/(376.73* fpat.pinr);
+	gcop= data.wlam* data.wlam* 2.0* M_PI/(376.73* fpat.pinr);
 	prad= fpat.pinr- fpat.ploss- fpat.pnlr;
 	gcon= gcop;
 	if( fpat.ipd != 0)
 	  gcon *= fpat.pinr/ prad;
   }
-  else
-	/*** For elementary current source ***/
-	if( fpat.ixtyp == 4)
-	{
-	  fpat.pinr=394.510* calc_data.xpr6*
-		calc_data.xpr6* data.wlam* data.wlam;
-	  gcop= data.wlam* data.wlam*2.0* PI/(376.73* fpat.pinr);
-	  prad= fpat.pinr- fpat.ploss- fpat.pnlr;
-	  gcon= gcop;
-	  if( fpat.ipd != 0)
-		gcon= gcon* fpat.pinr/ prad;
-	}
-	else
-	  /*** Incident field source ***/
-	{
-	  gcon=4.0* PI/(1.0+ calc_data.xpr6* calc_data.xpr6);
-	}
+  else if( fpat.ixtyp == 4) /*** For elementary current source ***/
+  {
+	fpat.pinr=394.510* calc_data.xpr6*
+	  calc_data.xpr6* data.wlam* data.wlam;
+	gcop= data.wlam* data.wlam*2.0* M_PI/(376.73* fpat.pinr);
+	prad= fpat.pinr- fpat.ploss- fpat.pnlr;
+	gcon= gcop;
+	if( fpat.ipd != 0)
+	  gcon= gcon* fpat.pinr/ prad;
+  }
+  else gcon=4.0* M_PI/(1.0+ calc_data.xpr6* calc_data.xpr6);
+  /*** Incident field source ***/
 
   phi  = fpat.phis - fpat.dph;
 
@@ -630,6 +624,8 @@ void rdpat(void)
 	rad_pattern[calc_data.fstep].min_gain[pol] =  10000.0;
 	rad_pattern[calc_data.fstep].max_gain_idx[pol] = 0;
 	rad_pattern[calc_data.fstep].min_gain_idx[pol] = 0;
+	rad_pattern[calc_data.fstep].max_gain_tht[pol] = 0;
+	rad_pattern[calc_data.fstep].max_gain_phi[pol] = 0;
   }
 
   /* Signal new rad pattern data */
@@ -640,7 +636,7 @@ void rdpat(void)
   for( kph = 1; kph <= fpat.nph; kph++ )
   {
 	phi += fpat.dph;
-	pha= phi* TA;
+	pha= phi* TORAD;
 	thet= fpat.thets - fpat.dth;
 
 	for( kth = 1; kth <= fpat.nth; kth++ )
@@ -653,11 +649,11 @@ void rdpat(void)
 		  gtk_widget_destroy( rdpattern_window );
 		fprintf( stderr, "xnec2c: rdpat(): Theta > 90 deg with ground specified\n"
 			"Please check RP card data and correct\n" );
-		stop( _("rdpat(): Theta > 90 deg with ground specified\n"\
+		stop( _("rdpat(): Theta > 90 deg with ground specified\n"
 			  "Please check RP card data and correct"), ERR_STOP );
 	  }
 
-	  tha= thet* TA;
+	  tha= thet* TORAD;
 	  if( gnd.ifar != 1)
 		ffld( tha, pha, &eth, &eph);
 	  else
@@ -695,27 +691,24 @@ void rdpat(void)
 		  if( fabs(dfaz) > fabs(dfaz2) )
 			dfaz= dfaz2;
 
-		  cdfaz= cos( dfaz* TA);
+		  cdfaz= cos( dfaz* TORAD);
 		  tstor1= ethm2- ephm2;
 		  tstor2=2.0* ephm* ethm* cdfaz;
-		  tilta=.5* atan2( tstor2, tstor1);
+		  tilta=atan2( tstor2, tstor1)/2.0;
 		  stilta= sin( tilta);
 		  tstor1= tstor1* stilta* stilta;
 		  tstor2= tstor2* stilta* cos( tilta);
 		  emajr2= -tstor1+ tstor2+ ethm2;
 		  eminr2= tstor1- tstor2+ ephm2;
-		  if( eminr2 < 0.0)
-			eminr2=0.0;
+		  if( eminr2 < 0.0)	eminr2=0.0;
 
 		  axrat= sqrt( eminr2/ emajr2);
-		  tilta= tilta* TD;
 		  if( axrat <= 1.0e-5)
 			isens= 1;
+		  else if( dfaz <= 0.0)
+			isens= 2;
 		  else
-			if( dfaz <= 0.0)
-			  isens= 2;
-			else
-			  isens= 3;
+			isens= 3;
 
 		} /* if( (ethm2 <= 1.0e-20) && (ephm2 <= 1.0e-20) ) */
 
@@ -752,28 +745,29 @@ void rdpat(void)
 		}
 
 		/* Save rad pattern gains */
-		rad_pattern[calc_data.fstep].gtot[idx] = (double)tstor1;
+		rad_pattern[calc_data.fstep].gtot[idx] = tstor1;
 
 		/* Save axial ratio, tilt and pol sense */
 		if( isens == 2 )
-		  rad_pattern[calc_data.fstep].axrt[idx] = (double)-axrat;
+		  rad_pattern[calc_data.fstep].axrt[idx] = -axrat;
 		else
-		  rad_pattern[calc_data.fstep].axrt[idx] = (double)axrat;
-		rad_pattern[calc_data.fstep].tilt[idx] = (double)(TA * tilta);
+		  rad_pattern[calc_data.fstep].axrt[idx] = axrat;
+		rad_pattern[calc_data.fstep].tilt[idx] = tilta;
 		rad_pattern[calc_data.fstep].sens[idx] = isens;
 
 		/* Find and save max value of gain and direction */
 		for( pol = 0; pol < NUM_POL; pol++ )
 		{
-		  gain = (double)rad_pattern[calc_data.fstep].gtot[idx] + 10.0 *
-			log10( Polarization_Factor( pol, calc_data.fstep, idx) );
+		  gain = rad_pattern[calc_data.fstep].gtot[idx] +
+			Polarization_Factor( pol, calc_data.fstep, idx);
+		  if( gain < -999.99 ) gain = -999.99;
 
 		  /* Find and save max value of gain and direction */
 		  if( rad_pattern[calc_data.fstep].max_gain[pol] < gain )
 		  {
 			rad_pattern[calc_data.fstep].max_gain[pol]     = gain;
-			rad_pattern[calc_data.fstep].max_gain_tht[pol] = (double)thet;
-			rad_pattern[calc_data.fstep].max_gain_phi[pol] = (double)phi;
+			rad_pattern[calc_data.fstep].max_gain_tht[pol] = thet;
+			rad_pattern[calc_data.fstep].max_gain_phi[pol] = phi;
 			rad_pattern[calc_data.fstep].max_gain_idx[pol] = idx;
 		  }
 
